function X = signals2(l,Fin,theta,phi)

%Generates sine signals of length l and resolution(sample rate) r with individual phases p(n), frequencies f(n) and
%amplitudes a(n). The generated signals are mixed and divided into four
%outputs with separate time delays tA, tB, tC, tD and uncorrelated random noise with amplitude nf, as four separate arrays.
%The signals are barrel cycled according to the delay


%l = Signal length in microseconds
%Fin=sample frequency in MHz
%Ts = Number of samples
Ts= (Fin*l);                                         %Number of samples


fc=45;                          %IF carrier frequency in MHz
f=[0 -3 -11 7];             %frequency components (relative to IF frequency)
a=[0.15 0.5 0.1 0.2];                    %normalized amplitudes (sum of elements in a should be <1

G = 1;                         %common gain factor (does not amplify system noise)
aA=1;%*2^15;                       %relative amplitude unbalance element A
aB=0.99;%*2^15;                  %relative amplitude unbalance element B
aC=1.03;%*2^15;                      %relative amplitude unbalance element C
aD=1.02;%*2^15;                     %relative amplitude unbalance element D
aE=0.98;%*2^15;
aF=0.97;%*2^15;
L =lambda3d(theta,phi,0.19); 
%dA= 0;%*fc/1575;                       %relative phase shift for element A
dB = -L(1);                                      %relative phase shift element B   
dC=-L(2);                                        %relative phase shift element C 
dD=-L(3);                                        %relative phase shift element D
dE=-L(4);                                        %relative phase shift element E  
dF = -L(5);                                      %relative phase shift element F
nf=1*10^(-2);         %noise floor amplitude (ADC noise + front-end noise)

%% Generate baseband signals
A=zeros(1,Ts);
B=zeros(1,Ts);
C=zeros(1,Ts);
D=zeros(1,Ts);
E=zeros(1,Ts);
F=zeros(1,Ts);

for n=500:1:Ts
    for m=1:1:length(f)
        A(n)=A(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin);%+dA);
        B(n)=B(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin+dB);
        C(n)=C(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin+dC);
        D(n)=D(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin+dD);
        E(n)=E(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin+dE);
        F(n)=F(n)+a(m)*sin(2*pi*(fc+f(m))*n/Fin+dF);
    end
end

%% add common gain,phase and noise
for n=1:1:Ts
    A(n)=G*(aA)*A(n)+nf*(rand-0.5)*2;
    B(n)=G*(aB)*B(n)+nf*(rand-0.5)*2;
    C(n)=G*(aC)*C(n)+nf*(rand-0.5)*2;
    D(n)=G*(aD)*D(n)+nf*(rand-0.5)*2;
    E(n)=G*(aE)*E(n)+nf*(rand-0.5)*2;
    F(n)=G*(aF)*F(n)+nf*(rand-0.5)*2;
end

%X=[A;B;C;D;E;F];
X=[A;B;C;D];

return
