--File: BreedingTb_sim.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: BreedingTb_sim.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
-- <Add Assumptions>
--Description: 
-- <Add Description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

library UTILSLIB;
use UTILSLIB.BasicTypes.all;

library TESTUTILSLIB;
use TESTUTILSLIB.TestbenchFunctions.all;
use TESTUTILSLIB.stdio_h.all;

library TESTUTILSSIM;
--Only use one of theese
use TESTUTILSSIM.GenTbFifo;  --Comment to use TbFifo instead of GenTbFifo
--use TESTUTILSSIM.TbFifo.all; --Unomment to use TbFifo instead of GenTbFifo

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library GENETICALGSIM;
use GENETICALGSIM.ComplexPrivateTb.all;

architecture sim of BreedingTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------

   -- DUT signals

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Comment this section to use the TbFifo instead GenTbFifo
   --    This code requires VHDL 2008
   --Define the record used by tcl
   type t_ElementType is record
      SimCase  : natural range 0 to 255;
      DataSets : natural;
   end record;
   
   --instansiate the tcl record 
   constant c_ElementTypeInit : t_ElementType := (SimCase  => 0,
                                                  DataSets => 0);
   
   --Create a Generic fifo used for commands from tcl
   --The input type is t_ElementType
   package GenFifo is new TESTUTILSSIM.GenTbFifo generic map(t_ElementType     => t_ElementType,
                                                             c_ElementTypeInit => c_ElementTypeInit);

   shared variable CmdFifo : GenFifo.t_GenTbFifo;  --Tcl command fifo.
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Uncomment to use TbFifo instead of GenTbFifo
   --    This code requires VHDL 2002
   --   shared variable CmdFifo  : t_TbFifo; --Tcl command fifo
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   -- DUT signals
   signal NewPopIn       : std_logic;
   signal ParrentA       : t_1xAT10;
   signal ParrentB       : t_1xAT10;
   signal RandomBits     : std_logic_vector(179 downto 0);
   signal IndividualOut  : t_1xAT10;
   signal NewPopOut      : std_logic;                  
   constant c_minValue10  : integer := -512;
   constant c_maxValue10  : integer := 511;

   -- Testbench signals 
   signal tcl_Cmd      : t_ElementType := c_ElementTypeInit;
   signal tcl_MainTrig : std_logic     := '0';  --Trigger the main case
   signal SimComplete  : std_logic     := '0';  --Informs tcl that the case is complete

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------


begin

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------

   -- Add instantiation of DUT
      i_Breeding : entity GENETICALGLIB.Breeding(rtl)
         port map (
         Rst_n          => Rst_n,
         Clk            => Clk,
         NewPopIn       => NewPopIn,
         ParrentA       => ParrentA,
         ParrentB       => ParrentB,
         RandomBits     => RandomBits,
         IndividualOut  => IndividualOut,
         NewPopOut      => NewPopOut
      );
   ----------------------------------------------------------------------------
   -- Processes
   ----------------------------------------------------------------------------

   p_Clk : Clock(Clk, ClkCycle);

   -- Receive commands from the Tcl scripts.
   p_RxCmd : process is
   begin
      wait until tcl_MainTrig = '1';
      log(ID_SEQUENCER, "Adding SimCase to CmdFifo:" & to_string(tcl_Cmd.SimCase,4));
      CmdFifo.push(tcl_Cmd);
      wait until tcl_MainTrig = '0';
   end process;


   --Main simulation process. Triggered by tcl.
   p_Sequencer : process is
      variable t        : time          := 0 ns;
      variable v_SimCmd : t_ElementType := c_ElementTypeInit;
      --DUT variables
      variable v_TestValues : t_IntArray(0 to 3,0 to 8);
      variable v_TestRandomBits : std_logic_vector (179 downto 0);
   begin
      -- Allow simulation to continue regardless of errors
      set_alert_stop_limit(ERROR, 10);   

      -- Verbosity control
      -- disable_log_msg(ID_BFM_DATA);
      -- disable_log_msg(ID_BFM_WAIT);
      -- disable_log_msg(ID_BFM);
      -- disable_log_msg(ID_POS_ACK); 

      -- Set expected test bench time stamp truncated warning
      increment_expected_alerts(TB_WARNING, 1);

      log(ID_LOG_HDR, "Start of simulation.");

      while true loop
         
         -- Wait for Sim Cmd in FIFO, then read Sim Cmd 
         if CmdFifo.fifo_depth = 0 then
            wait until tcl_MainTrig = '1';
            wait until tcl_MainTrig = '0';
         end if;

         t := now;
         v_SimCmd := CmdFifo.pop;

         --Inform tcl that the case is running
         SimComplete <= '0';

         case v_SimCmd.SimCase is
            --
            -- Random data testing
            --
            when 1 =>
               log(ID_LOG_HDR, "TestCase " & to_string(v_SimCmd.SimCase,4) & ": Testing with random data.");
               
               for i in 0 to 1000 loop
                  v_TestRandomBits := random(180);
                  
                  for j in 0 to 3 loop
                     for k in 0 to 8 loop
                        v_TestValues(j,k) := random(c_minValue10,c_maxValue10); 
                     end loop;
                  end loop;
                  
                  wait until rising_edge(Clk);
                   
                  TestBreeding(
                     NewPopIn          => NewPopIn,
                     ParrentA          => ParrentA,
                     ParrentB          => ParrentB,
                     RandomBits        => RandomBits,
                     IndividualOut     => IndividualOut,
                     NewPopOut         => NewPopOut,
                     v_TestValues      => v_TestValues,
                     v_TestRandomBits  => v_TestRandomBits);
               end loop;



            --
            -- Force the reset from the tcl
            --
            when 254 =>
               log(ID_LOG_HDR, "TestCase " & to_string(v_SimCmd.SimCase,4) & ": Resetting design.");
               wait until rising_edge(Clk);
               Rst_n <= '0';
               wait for 10*ClkCycle;
               Rst_n <= '1';
               wait until rising_edge(Clk);

            --
            -- Display summary
            --
            when 255 =>
               wait until rising_edge(Clk);
               report_alert_counters(VOID);

               --Inform tcl that the case is compete
               SimComplete <= '1';

            when others =>
               log(ID_SEQUENCER, "Test not implemented: " & to_string(v_SimCmd.SimCase,4));

         end case;

         -- Report time spent in a testcase
         if now - t < 5000 ns then
            log (ID_SEQUENCER, "Test performed in " &  integer'image(integer(((now - t) / 1 ns))) & " ns");
         elsif now - t < 5000 us then
            log (ID_SEQUENCER, "Test performed in " &  integer'image(integer(((now - t) / 1 us))) & " us");
         else
            log (ID_SEQUENCER, "Test performed in " &  integer'image(integer(((now - t) / 1 ms))) & " ms");
         end if;
         
      end loop;
   end process p_Sequencer;

   
   

end sim;
