--File: ComplexPrivateTb_bdy.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: ComplexPrivateTb_bdy.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
--<Add assumptions>
--Description: 
--<Add description>

package body ComplexPrivateTb is

--Testing the vector multiplier unit
procedure TestVectorMul1610(
   signal Hermit16 : out t_1xAT16;
   signal Vector10 : out t_1xAT10;
   signal Result   : in t_Complex20;
   signal DataValid: out std_logic ;
   signal ResultValid: in std_logic ;
   variable v_TestValues : in t_IntArray
) is
   variable v_checkRes   : boolean;
   
   variable v_Are : integer;
   variable v_Aim : integer;
   variable v_Bre : integer;
   variable v_Bim : integer;
   variable v_ExpResIm : integer;
   variable v_ExpResRe : integer;
   variable v_ExpResReSigned :signed(Hermit16(0).im'length+Vector10(0).im'length-1 downto 0);
   variable v_ExpResImSigned :signed(Hermit16(0).im'length+Vector10(0).im'length-1 downto 0);

begin
   
   wait for ClkCycle;
   for i in 0 to 8 loop
      Hermit16(i).re <= to_signed(v_TestValues(0,i),16); 
      Hermit16(i).im <= to_signed(v_TestValues(1,i),16);
      Vector10(i).re <= to_signed(v_TestValues(2,i),10);
      Vector10(i).im <= to_signed(v_TestValues(3,i),10);
   end loop;
   
   DataValid <= '1';
   wait for ClkCycle;
   DataValid <= '0';
   wait until ResultValid = '1';
   wait for 1 ns;
   
   v_ExpResIm := 0; 
   v_ExpResRe := 0;
   
   for i in 0 to 8 loop      
      v_Are := v_TestValues(0,i);
      v_Aim := v_TestValues(1,i);
      v_Bre := v_TestValues(2,i);
      v_Bim := v_TestValues(3,i);

      v_ExpResImSigned := to_signed(v_Are*v_Bim + v_Aim*v_Bre,Hermit16(0).im'length+Vector10(0).im'length);
      v_ExpResReSigned := to_signed(v_Are*v_Bre - v_Aim*v_Bim,Hermit16(0).im'length+Vector10(0).im'length);
      
      v_ExpResIm := v_ExpResIm + to_integer(v_ExpResImSigned(Hermit16(0).im'length+Vector10(0).im'length-1 downto Vector10(0).im'length));
      v_ExpResRe := v_ExpResRe + to_integer(v_ExpResReSigned(Hermit16(0).im'length+Vector10(0).im'length-1 downto Vector10(0).im'length));
   end loop;
      
   v_checkRes := check_value(to_integer(Result.re),v_ExpResRe,warning,"vector multiplication real part");
   v_checkRes := check_value(to_integer(Result.im),v_ExpResIm,warning,"vector multiplication imaginary part");
end procedure;



--for testing
procedure TruncatedIntegerComplexVectorMul(
   variable v_InputVectors : in t_IntArray;
   variable v_ExpResIm : out integer;   
   variable v_ExpResRe : out integer;
   constant c_VectorLengthA : integer;
   constant c_VectorLengthB : integer 
) is
   variable v_ExpResImSigned : signed(c_VectorLengthB+c_VectorLengthA-1 downto 0);
   variable v_ExpResReSigned : signed(c_VectorLengthB+c_VectorLengthA-1 downto 0);
   variable v_Are : integer;
   variable v_Aim : integer;
   variable v_Bre : integer;
   variable v_Bim : integer;
begin
   v_ExpResIm := 0; 
   v_ExpResRe := 0;
   
   for i in 0 to 8 loop      
      v_Are := v_InputVectors(0,i);
      v_Aim := v_InputVectors(1,i);
      v_Bre := v_InputVectors(2,i);
      v_Bim := v_InputVectors(3,i);

      v_ExpResImSigned := to_signed(v_Are*v_Bim + v_Aim*v_Bre,c_VectorLengthA+c_VectorLengthB);
      v_ExpResReSigned := to_signed(v_Are*v_Bre - v_Aim*v_Bim,c_VectorLengthA+c_VectorLengthB);
      
      v_ExpResIm := v_ExpResIm + to_integer(v_ExpResImSigned(c_VectorLengthA+c_VectorLengthB-1 downto c_VectorLengthB));
      v_ExpResRe := v_ExpResRe + to_integer(v_ExpResReSigned(c_VectorLengthA+c_VectorLengthB-1 downto c_VectorLengthB));
   end loop;

end procedure;

--Testing the vector multiplier unit
procedure TestVectorMul1616(
   signal Hermit16 : out t_1xAT16;
   signal Vector16 : out t_1xAT16;
   signal Result   : in t_Complex20;
   signal DataValid: out std_logic ;
   signal ResultValid: in std_logic ;
   variable v_TestValues : in t_IntArray
) is
   variable v_checkRes   : boolean;
   variable v_ExpResIm : integer;
   variable v_ExpResRe : integer;
begin
   
   wait for ClkCycle;
   for i in 0 to 8 loop
      Hermit16(i).re <= to_signed(v_TestValues(0,i),Hermit16(i).re'length); 
      Hermit16(i).im <= to_signed(v_TestValues(1,i),Hermit16(i).re'length);
      Vector16(i).re <= to_signed(v_TestValues(2,i),Vector16(i).re'length);
      Vector16(i).im <= to_signed(v_TestValues(3,i),Vector16(i).re'length);
   end loop;
   
   DataValid <= '1';
   wait for ClkCycle;
   DataValid <= '0';
   wait until ResultValid = '1';
   
   TruncatedIntegerComplexVectorMul(
   v_InputVectors    => v_TestValues,
   v_ExpResIm        => v_ExpResIm,
   v_ExpResRe        => v_ExpResRe,
   c_VectorLengthA   => Hermit16(0).re'length,
   c_VectorLengthB   => Vector16(0).re'length);
      
   v_checkRes := check_value(to_integer(Result.re),v_ExpResRe,warning,"vector multiplication real part");
   v_checkRes := check_value(to_integer(Result.im),v_ExpResIm,warning,"vector multiplication imaginary part");
end procedure;

--Testing one multiplication with two 16 bit inputs
   procedure TestSingleComplexMul(
      signal A   : in t_Complex16;
      signal B   : in t_Complex10;
      signal Res : in  t_Complex16             
      ) is
      
      variable v_ExpResIm : integer;
      variable v_ExpResRe : integer;
      variable v_checkRes   : boolean;
      variable v_ExpResReSigned :signed(A.im'length+B.im'length-1 downto 0);

      variable v_ExpResImSigned :signed(A.im'length+B.im'length-1 downto 0);

      variable v_Are : signed(A.re'range);
      variable v_Aim : signed(A.im'range);
      variable v_Bre : signed(B.re'range);
      variable v_Bim : signed(B.im'range);

begin           
      wait for 3*ClkCycle;
      
      v_Are := A.re;
      v_Aim := A.im;
      v_Bre := B.re;
      v_Bim := B.im;

      v_ExpResImSigned := to_signed(to_integer(v_Are)*to_integer(v_Bim) + to_integer(v_Aim)*to_integer(v_Bre),A.im'length+B.im'length);
      v_ExpResReSigned := to_signed(to_integer(v_Are)*to_integer(v_Bre) - to_integer(v_Aim)*to_integer(v_Bim),A.im'length+B.im'length);
      
      v_ExpResIm := to_integer(v_ExpResImSigned(A.im'length+B.im'length-1 downto B.im'length));
      v_ExpResRe := to_integer(v_ExpResReSigned(A.im'length+B.im'length-1 downto B.im'length));
      

      --log(ID_LOG,"Imag: " & to_string(to_integer(v_Are)) & " * " & to_string(to_integer(v_Bim)) & " + " & to_string(to_integer(v_Aim)) & " * " & to_string(to_integer(v_Bre)));
      --log(ID_LOG,"Real: " & to_string(to_integer(v_Are)) & " * " & to_string(to_integer(v_Bre)) & " - " & to_string(to_integer(v_Aim)) & " * " & to_string(to_integer(v_Bim)));

      v_checkRes := check_value(to_integer(Res.re),v_ExpResRe,warning,"multiplication real part");
      v_checkRes := check_value(to_integer(Res.im),v_ExpResIm,warning,"multiplication imaginary part");
end procedure;




--Todo: Add comment
   procedure TestComplexMul(
      signal A   : out t_Complex16;
      signal B   : out t_Complex10;
      signal Res : in  t_Complex16   
   ) is      
      
      variable v_Are : signed(A.re'range);
      variable v_Aim : signed(A.im'range);
      variable v_Bre : signed(B.re'range);
      variable v_Bim : signed(B.im'range);

   begin
      wait until rising_edge(Clk);
      wait for 3*ClkCycle;   
      
      -- Testing multiplier with random data
      for i in 0 to 1000 loop
      
         v_Are := signed(random(A.re'length)); 
         v_Bre := signed(random(B.re'length));
         v_Aim := signed(random(A.im'length)); 
         v_Bim := signed(random(B.im'length));
         
         A.re <= v_Are;
         A.im <= v_Aim;
         B.re <= v_Bre;
         B.im <= v_Bim;
         
         TestSingleComplexMul(A => A,
                              B => B,
                              Res => Res);
       
      end loop;
         
         --max positive value
         A.re <= to_signed(32767,A.re'length);
         A.im <= to_signed(32767,A.re'length);
         B.re <= to_signed(511,B.re'length);
         B.im <= to_signed(511,B.re'length);

         TestSingleComplexMul(A,B,Res);
         
         --max negative data
         A.re <= to_signed(-32768,A.re'length);
         A.im <= to_signed(-32768,A.re'length);
         B.re <= to_signed(-512,B.re'length);
         B.im <= to_signed(-512,B.re'length);

         TestSingleComplexMul(A,B,Res);


   end procedure;

   --for cost function
   procedure HVecMatrixVecMultiplication(
      variable v_InputIntegerMatrix : in t_IntArray;
      variable v_ExpResIm : out integer;
      variable v_ExpResRe : out integer;
      constant c_VectorLength : integer;
      constant c_MatrixLength : integer 
   ) is
      variable v_TestValues : t_IntArray(0 to 3,0 to c_AT-1);
      variable v_MidResults : t_IntArray(0 to 1,0 to c_AT-1);
      variable v_MidResultRe : integer;
      variable v_MidResultIm : integer;
      variable v_MidResultSignedRe : signed(19 downto 0);
      variable v_MidResultSignedIm : signed(19 downto 0);
   begin
   
   for i in 0 to c_AT-1 loop   
      v_TestValues(0,i) := v_InputIntegerMatrix(0,i);   
      v_TestValues(1,i) := -v_InputIntegerMatrix(1,i);   
   end loop;

   for i in 1 to c_AT loop
      
      for k in 0 to c_AT-1 loop
         v_TestValues(2,k) := v_InputIntegerMatrix(2*i,k);
         v_TestValues(3,k) := v_InputIntegerMatrix(1+2*i,k);
      end loop;
      
      TruncatedIntegerComplexVectorMul(
         v_InputVectors    => v_TestValues,
         v_ExpResIm        => v_MidResultIm,
         v_ExpResRe        => v_MidResultRe,
         c_VectorLengthA   => c_MatrixLength,
         c_VectorLengthB   => c_VectorLength);
      
      v_MidResultSignedRe := to_signed(v_MidResultRe,v_MidResultSignedRe'length);
      v_MidResultSignedIm := to_signed(v_MidResultIm,v_MidResultSignedIm'length);
      v_MidResults(0,i-1) := to_integer(v_MidResultSignedRe(v_MidResultSignedRe'length-1 downto v_MidResultSignedRe'length - c_MatrixLength));
      v_MidResults(1,i-1) := to_integer(v_MidResultSignedIm(v_MidResultSignedRe'length-1 downto v_MidResultSignedRe'length - c_MatrixLength));
   
      --log(ID_LOG,"Imag: " & to_string(to_integer(v_MidResultSignedIm(v_MidResultSignedRe'length-1 downto v_MidResultSignedRe'length - c_MatrixLength))) & " Real:" & to_string(to_integer(v_MidResultSignedRe(v_MidResultSignedRe'length-1 downto v_MidResultSignedRe'length - c_MatrixLength))));
   end loop;
   for k in 0 to c_AT-1 loop 
      v_TestValues(0,k) := v_InputIntegerMatrix(0,k);   
      v_TestValues(1,k) := v_InputIntegerMatrix(1,k);   
      v_TestValues(2,k) := v_MidResults(0,k);   
      v_TestValues(3,k) := v_MidResults(1,k);
   end loop;
      
   TruncatedIntegerComplexVectorMul(
     v_InputVectors    => v_TestValues,
     v_ExpResIm        => v_ExpResIm,
     v_ExpResRe        => v_ExpResRe,
     c_VectorLengthA   => c_MatrixLength,
     c_VectorLengthB   => c_VectorLength);


   end procedure;
   
   procedure TestHVecMatrixVecMultiplication( 
      variable v_InputIntegerMatrix : in t_IntArray;
      signal InputMatrix : out t_ATxAT16;
      signal InputVector : out t_1xAT10;
      signal DataValid   : out std_logic;
      signal Result      : in t_Complex20;
      signal ResultValid : in std_logic 
   ) is
      
   variable v_checkRes   : boolean;
   variable v_ExpResIm : integer;
   variable v_ExpResRe : integer;
begin
   
   wait for 1*ClkCycle;
   for col in 0 to 8 loop
      InputVector(col).re <= to_signed(v_InputIntegerMatrix(0,col),InputVector(0).re'length);
      InputVector(col).im <= to_signed(v_InputIntegerMatrix(1,col),InputVector(0).re'length);
      for rad in 0 to 8 loop
         InputMatrix(rad)(col).re <= to_signed(v_InputIntegerMatrix(rad*2+2,col),InputMatrix(0)(0).re'length); 
         InputMatrix(rad)(col).im <= to_signed(v_InputIntegerMatrix(rad*2+3,col),InputMatrix(0)(0).re'length);
      end loop;
   end loop;
   
   DataValid <= '1';
   wait for ClkCycle;
   DataValid <= '0';
   wait until ResultValid = '1';
   wait for 0.1*ClkCycle;
   
   HVecMatrixVecMultiplication(
   v_InputIntegerMatrix => v_InputIntegerMatrix,
   v_ExpResIm           => v_ExpResIm,
   v_ExpResRe           => v_ExpResRe,
   c_VectorLength       => InputVector(0).re'length,
   c_MatrixLength       => InputMatrix(0)(0).re'length);
      
   v_checkRes := check_value(to_integer(Result.re),v_ExpResRe,warning,"vector multiplication real part");
   v_checkRes := check_value(to_integer(Result.im),v_ExpResIm,warning,"vector multiplication imaginary part");

end procedure;



--CostFunction calculation of expercted result
procedure CostFunction(
   variable v_InputIntegerMatrix: in t_IntArray;
   variable v_ExpResIm : out integer;
   variable v_ExpResRe : out integer;
   constant c_VectorLengthA : integer; --16
   constant c_VectorLengthB : integer --10
) is
   Variable v_PartialResult1Re : integer;
   Variable v_PartialResult1Im : integer;      
   Variable v_PartialResult2Re : integer;
   Variable v_PartialResult2Im : integer;      
   Variable v_PartialResult3Re : integer;
   Variable v_PartialResult3Im : integer;
   Variable v_PartialResult1ReSigned : signed(19 downto 0); 
   Variable v_PartialResult1ImSigned : signed(19 downto 0); 
   Variable v_PartialResult2ReSigned : signed(19 downto 0); 
   Variable v_PartialResult2ImSigned : signed(19 downto 0); 
   Variable v_PartialResult3ReSigned : signed(19 downto 0); 
   Variable v_PartialResult3ImSigned : signed(19 downto 0); 
   Variable v_PartialResult4ReSigned : signed(19 downto 0); 
   Variable v_PartialResult4ImSigned : signed(19 downto 0); 
   
   variable v_InputVectors     : t_IntArray(0 to 3, 0 to c_AT-1);      
begin
   HVecMatrixVecMultiplication(
      v_InputIntegerMatrix => v_InputIntegerMatrix,
      v_ExpResIm           => v_PartialResult1Im,
      v_ExpResRe           => v_PartialResult1Re,
      c_VectorLength       => c_VectorLengthB,
      c_MatrixLength       => c_VectorLengthA);
   for i in 0 to c_AT-1 loop
      v_InputVectors(0,i) := v_InputIntegerMatrix (20,i);
      v_InputVectors(1,i) := -v_InputIntegerMatrix(21,i);
      v_InputVectors(2,i) := v_InputIntegerMatrix (0 ,i);
      v_InputVectors(3,i) := v_InputIntegerMatrix (1 ,i);
   end loop;

   TruncatedIntegerComplexVectorMul(
      v_InputVectors    => v_InputVectors,
      v_ExpResIm        => v_PartialResult2Im,
      v_ExpResRe        => v_PartialResult2Re,
      c_VectorLengthA   => c_VectorLengthA,
      c_VectorLengthB   => c_VectorLengthB);

   for i in 0 to c_AT-1 loop
      v_InputVectors(0,i) := v_InputIntegerMatrix  (20,i);
      v_InputVectors(1,i) := v_InputIntegerMatrix  (21,i);
      v_InputVectors(2,i) := v_InputIntegerMatrix  (0 ,i);
      v_InputVectors(3,i) := -v_InputIntegerMatrix (1 ,i);
   end loop;

   TruncatedIntegerComplexVectorMul(
      v_InputVectors    => v_InputVectors,
      v_ExpResIm        => v_PartialResult3Im,
      v_ExpResRe        => v_PartialResult3Re,
      c_VectorLengthA   => c_VectorLengthA,
      c_VectorLengthB   => c_VectorLengthB);
   
   log(ID_LOG,"P1.re: " & to_string(v_InputIntegerMatrix(22,0)) & "  P1.im:" & to_string(v_InputIntegerMatrix(22,1)));
   log(ID_LOG,"SloMul1.re: " & to_string(v_PartialResult2Re) & "  SloMul1.im:" & to_string(v_PartialResult2Im));
   log(ID_LOG,"SloMul2.re: " & to_string(v_PartialResult3Re) & "  SloMul2.im:" & to_string(v_PartialResult3Im));
   log(ID_LOG,"MatrMul.re: " & to_string(v_PartialResult1Re) & "  MatrMul.im:" & to_string(v_PartialResult1Im));
   
   v_PartialResult1ReSigned := to_signed(v_PartialResult1Re,v_PartialResult1ReSigned'length); 
   v_PartialResult1ImSigned := to_signed(v_PartialResult1Im,v_PartialResult1ReSigned'length);
   v_PartialResult2ReSigned := to_signed(v_PartialResult2Re,v_PartialResult1ReSigned'length);
   v_PartialResult2ImSigned := to_signed(v_PartialResult2Im,v_PartialResult1ReSigned'length);
   v_PartialResult3ReSigned := to_signed(v_PartialResult3Re,v_PartialResult1ReSigned'length);
   v_PartialResult3ImSigned := to_signed(v_PartialResult3Im,v_PartialResult1ReSigned'length);
   v_PartialResult4ReSigned := to_signed(v_InputIntegerMatrix(22,0),v_PartialResult1ReSigned'length);
   v_PartialResult4ImSigned := to_signed(v_InputIntegerMatrix(22,1),v_PartialResult1ReSigned'length);             
   
   v_PartialResult1ReSigned  :=  v_PartialResult1ReSigned + to_signed(to_integer(v_PartialResult4ReSigned(v_PartialResult1ReSigned'length-1 downto 4)),20);
   v_PartialResult1ImSigned  :=  v_PartialResult1ImSigned + to_signed(to_integer(v_PartialResult4ImSigned(v_PartialResult1ReSigned'length-1 downto 4)),20);
   v_PartialResult2ReSigned  :=  to_signed(to_integer(v_PartialResult3ReSigned(v_PartialResult1ReSigned'length-1 downto 4)),20) + to_signed(to_integer(v_PartialResult2ReSigned(v_PartialResult1ReSigned'length-1 downto 4)),20);
   v_PartialResult2ImSigned  :=  to_signed(to_integer(v_PartialResult3ImSigned(v_PartialResult1ReSigned'length-1 downto 4)),20) + to_signed(to_integer(v_PartialResult2ImSigned(v_PartialResult1ReSigned'length-1 downto 4)),20);
   
   v_ExpResRe := to_integer(v_PartialResult1ReSigned - v_PartialResult2ReSigned) ;  
   v_ExpResIm := to_integer(v_PartialResult1ImSigned - v_PartialResult2ImSigned) ;  

   
end procedure;


procedure TestCostFunction( 
      variable v_InputIntegerMatrix : in t_IntArray;
      signal InputMatrix : out t_ATxAT16;
      signal InputVectorW : out t_1xAT10;
      signal InputVectorC : out t_1xAT16;
      signal DataValid   : out std_logic;
      signal P1          : out t_Complex20;
      signal Result      : in t_Complex20;
      signal Ready       : in std_logic;
      signal ResultValid : in std_logic 
   ) is
      
   variable v_checkRes   : boolean;
   variable v_ExpResIm : integer;
   variable v_ExpResRe : integer;
begin
   wait for ClkCycle;
   for col in 0 to c_AT-1 loop
      InputVectorW(col).re <= to_signed(v_InputIntegerMatrix(0,col),InputVectorW(0).re'length);
      InputVectorW(col).im <= to_signed(v_InputIntegerMatrix(1,col),InputVectorW(0).re'length);
      for rad in 0 to c_AT-1 loop
         InputMatrix(rad)(col).re <= to_signed(v_InputIntegerMatrix(rad*2+2,col),InputMatrix(0)(0).re'length); 
         InputMatrix(rad)(col).im <= to_signed(v_InputIntegerMatrix(rad*2+3,col),InputMatrix(0)(0).re'length);
      end loop;
      InputVectorC(col).re <= to_signed(v_InputIntegerMatrix(2*c_AT+2,col),InputVectorC(0).re'length);
      InputVectorC(col).im <= to_signed(v_InputIntegerMatrix(2*c_AT+3,col),InputVectorC(0).re'length);
   end loop;
   P1.re <= to_signed(v_InputIntegerMatrix(2*c_AT+4,0),P1.re'length);
   P1.im <= to_signed(v_InputIntegerMatrix(2*c_AT+4,1),P1.re'length);
  
   
   DataValid <= '1';
   wait for ClkCycle;
   DataValid <= '0';
   wait until ready ='1';
   
   InputMatrix  <= (others => (others => (others => (others => '1') ) ) );
   InputVectorW <= (others => (others => (others => '1') ) );
   InputVectorC <= (others => (others => (others => '1') ) );

   wait until ResultValid = '1';
   
   CostFunction(
   v_InputIntegerMatrix => v_InputIntegerMatrix,
   v_ExpResIm           => v_ExpResIm,
   v_ExpResRe           => v_ExpResRe,
   c_VectorLengthA      => InputVectorC(0).re'length,
   c_VectorLengthB      => InputVectorW(0).re'length);
      
   v_checkRes := check_value(to_integer(Result.re),v_ExpResRe,warning,"Cost Function real part");
   v_checkRes := check_value(to_integer(Result.im),v_ExpResIm,warning,"Cost Function imaginary part");

end procedure;

procedure SortIndividuals(
   variable v_InputIntegerMatrix: in t_IntArray;
   variable v_OutputIntegerMatrix: out t_IntArray
) is
   variable v_TempInt : integer;
   variable v_InputIntegerMatrixInt : t_IntArray(0 to 47,0 to c_AT); 

begin
   v_InputIntegerMatrixInt := v_InputIntegerMatrix;
   for i in 1 to 24 loop
      for row in 0 to 22 loop
         if v_InputIntegerMatrixInt(2*row,c_AT) > v_InputIntegerMatrixInt(2*(row+1),c_AT) then
            for k in 0 to c_AT loop
               
               v_TempInt := v_InputIntegerMatrixInt((row+1)*2,k);
               v_InputIntegerMatrixInt((row+1)*2,k) := v_InputIntegerMatrixInt(row*2,k);
               v_InputIntegerMatrixInt(row*2,k)   := v_TempInt;
               
               
               v_TempInt := v_InputIntegerMatrixInt((row+1)*2+1,k);
               v_InputIntegerMatrixInt((row+1)*2+1,k) := v_InputIntegerMatrixInt(row*2+1,k);
               v_InputIntegerMatrixInt(row*2+1,k)   := v_TempInt;
            
            end loop;
         end if;
      end loop;
   end loop; 
   
   for i in 0 to 23 loop
      for k in 0 to c_AT-1 loop
         v_OutputIntegerMatrix(i,k) := v_InputIntegerMatrixInt(i,k);
      end loop;
   end loop;

end procedure;


procedure TestInsertionSort(
   variable v_InputIntegerMatrix : in t_IntArray;
   signal OutputWeights          : in t_1xAT10;
   signal DataValid              : in std_logic;
   signal InputWeights           : out t_1xAT10;
   signal InputCost              : out t_Complex20;
   signal InputCostAddress       : out std_logic_vector(6 downto 0);
   signal InputWeightAddress     : out std_logic_vector(6 downto 0);
   signal NewPopIn               : out std_logic  
)is
   variable v_CheckRes   : boolean;
   variable v_Element      : integer;
   variable v_OutputIntegerMatrix: t_IntArray(0 to 23, 0 to c_AT-1);
begin
   
   wait until rising_edge(Clk);
   
   for i in 0 to 23 loop
      if i = 0 then
         NewPopIn <= '1';
      else
         NewPopIn <= '0';
      end if;
      
      InputCostAddress       <= std_logic_vector(to_unsigned(i,7));
      InputWeightAddress     <= std_logic_vector(to_unsigned(i,7));
      
      InputCost.re <= to_signed(v_InputIntegerMatrix(i*2,c_AT),InputCost.re'length);
      InputCost.im <= to_signed(v_InputIntegerMatrix(i*2+1,c_AT),InputCost.im'length);
      
      for col in 0 to c_AT-1 loop
         InputWeights(col).re <= to_signed(v_InputIntegerMatrix(i*2,col),InputWeights(0).re'length);    
         InputWeights(col).im <= to_signed(v_InputIntegerMatrix(i*2+1,col),InputWeights(0).im'length);
      end loop;
      
      wait until rising_edge(Clk);
   
   end loop;
   
   SortIndividuals(
      v_InputIntegerMatrix  => v_InputIntegerMatrix,
      v_OutputIntegerMatrix => v_OutputIntegerMatrix);
   
   NewPopIn <= '1';
   
   wait until rising_edge(Clk);
   NewPopIn <= '0';
   
   wait until DataValid = '1' and rising_edge(Clk);
   v_Element := 0;   
   while (DataValid = '1') loop
      for i in 0 to c_AT-1 loop
         if v_Element < 12 then
            v_CheckRes := check_value(to_integer(OutputWeights(i).re),v_OutputIntegerMatrix(23-(v_Element*2+1),i),warning,"RealWeight");
            v_CheckRes := check_value(to_integer(OutputWeights(i).im),v_OutputIntegerMatrix(23-(v_Element*2),i),warning,"ImagWeight");
         end if;
      end loop;
      wait until rising_edge(Clk);
      v_Element := v_Element +1 ;
   end loop;

end procedure;

--Mutation test procedure
procedure TestMutation(
   signal WeightsOut       : in t_1xAT10;
   signal RandomBits       : out std_logic_vector (1259 downto 0);
   signal WeightsIn        : out t_1xAT10;
   signal MutationRate     : out t_IntegerArray;
   variable v_TestValues   : in t_IntArray;
   variable v_TestRandomBits       : in std_logic_vector (1259 downto 0);
   variable v_MutationRate           : in t_IntegerArray
) is
  variable v_checkRes   : boolean;
  variable v_WeightsIn  : t_1xAT10;
  variable v_BitFail    :std_logic;
  variable v_MutateImag : std_logic;
  variable v_MutateReal : std_logic;
  variable v_PartResImag : std_logic;
  variable v_PartResReal : std_logic;

begin
   
   wait until rising_edge(Clk);
   
   for i in 0 to c_AT-1 loop
      WeightsIn(i).re <= to_signed(v_TestValues(0,i),10);
      v_WeightsIn(i).re := to_signed(v_TestValues(0,i),10);
      WeightsIn(i).im <= to_signed(v_TestValues(1,i),10);
      v_WeightsIn(i).im := to_signed(v_TestValues(1,i),10);
   end loop;
   MutationRate <= v_MutationRate;
   RandomBits <= v_TestRandomBits;

   wait until rising_edge(Clk);
   wait until rising_edge(Clk);
   wait for 1 ps;
    
   for i in 0 to c_AT-1 loop
      v_BitFail := '0';
      for j in c_NumBitsLow -1 downto 0 loop
         v_MutateImag := '1';
         v_MutateReal := '1';
         
         for k in 0 to v_MutationRate(0)-1 loop
            v_MutateImag := v_MutateImag and v_TestRandomBits((i*10+j)*2*7+k);
            v_MutateReal := v_MutateReal and v_TestRandomBits((i*10+j)*2*7+k+7);
         end loop;
         v_PartResImag := v_WeightsIn(i).im(j) xor v_MutateImag;
         v_PartResReal := v_WeightsIn(i).re(j) xor v_MutateReal;
         --log(ID_LOG,"Mutated.re: " & to_string(v_PartResReal) & "  Mutated.im:" & to_string(v_PartResImag));
         if (v_WeightsIn(i).im(j) xor v_MutateImag) /= WeightsOut(i).im(j) or (v_WeightsIn(i).re(j) xor v_MutateReal) /= WeightsOut(i).re(j) then    
            v_BitFail := '1';
         end if;
      end loop;
      v_checkRes := check_value(v_BitFail,'0',warning,"Mutation ok?");  
   end loop;
end procedure;

--For testing of the breeding module
procedure TestBreeding(
   signal NewPopIn          : out std_logic;
   signal ParrentA          : out t_1xAT10;
   signal ParrentB          : out t_1xAT10;
   signal RandomBits        : out std_logic_vector (179 downto 0);
   signal IndividualOut     : in t_1xAT10;
   signal NewPopOut         : in std_logic;
   variable v_TestValues      : in t_IntArray;
   variable v_TestRandomBits  : std_logic_vector (179 downto 0)
) is
  variable v_checkRes   : boolean;
  variable v_ParrentA  : t_1xAT10;
  variable v_ParrentB  : t_1xAT10;
  variable v_BitFail    :std_logic;

begin
   
   wait until rising_edge(Clk);
   
   for i in 0 to c_AT-1 loop
      ParrentA(i).re <= to_signed(v_TestValues(0,i),10);
      v_ParrentA(i).re := to_signed(v_TestValues(0,i),10);
      ParrentA(i).im <= to_signed(v_TestValues(1,i),10);
      v_ParrentA(i).im := to_signed(v_TestValues(1,i),10);
      
      ParrentB(i).re <= to_signed(v_TestValues(2,i),10);
      v_ParrentB(i).re := to_signed(v_TestValues(2,i),10);
      ParrentB(i).im <= to_signed(v_TestValues(3,i),10);
      v_ParrentB(i).im := to_signed(v_TestValues(3,i),10);
   end loop;
   RandomBits <= v_TestRandomBits;
   NewPopIn  <= '1';
   
   wait until rising_edge(Clk);
   NewPopIn <= '0';
   wait until NewPopOut = '1' and rising_edge(Clk);
   
   for mode in 1 to 4 loop
      for i in 0 to c_AT-1 loop
         v_BitFail := '0';
         for j in c_NumBitsLow -1 downto 0 loop
            --ParrentA
            if mode = 1 then
               if v_ParrentA(i).im(j) /= IndividualOut(i).im(j) or v_ParrentA(i).re(j) /= IndividualOut(i).re(j) then    
                  v_BitFail := '1';
               end if;
            --ParrentB
            elsif mode = 2 then
               if v_ParrentB(i).im(j) /= IndividualOut(i).im(j) or v_ParrentB(i).re(j) /= IndividualOut(i).re(j) then    
                  v_BitFail := '1';
               end if;
            --Child 1
            elsif mode = 3 then
               if v_TestRandomBits((i*c_NumBitsLow + j)*2) = '1' then   
                  if IndividualOut(i).re(j) /= v_ParrentA(i).re(j) then
                     v_BitFail := '1';
                  end if;   
               else
                  if IndividualOut(i).re(j) /= v_ParrentB(i).re(j) then
                     v_BitFail := '1';
                  end if;   
               end if;
               
               if v_TestRandomBits((i*c_NumBitsLow + j)*2+1) = '1' then   
                  if IndividualOut(i).im(j) /= v_ParrentA(i).im(j) then
                     v_BitFail := '1';
                  end if;
               else
                  if IndividualOut(i).im(j) /= v_ParrentB(i).im(j) then
                     v_BitFail := '1';
                  end if;   
               end if;
            --Child 2
            else
               if v_TestRandomBits((i*c_NumBitsLow + j)*2) = '0' then   
                  if IndividualOut(i).re(j) /= v_ParrentA(i).re(j) then
                     v_BitFail := '1';
                  end if;   
               else
                  if IndividualOut(i).re(j) /= v_ParrentB(i).re(j) then   
                     v_BitFail := '1';
                  end if;   
               end if;
               
               if v_TestRandomBits((i*c_NumBitsLow + j)*2+1) = '0' then   
                  if IndividualOut(i).im(j) /= v_ParrentA(i).im(j) then
                     v_BitFail := '1';
                  end if;
               else
                  if IndividualOut(i).im(j) /= v_ParrentB(i).im(j) then
                     v_BitFail := '1';
                  end if;   
               end if;
            end if;
         end loop;
         v_checkRes := check_value('0',v_BitFail,warning,"Checking if breeding is done correctly");  
      end loop;
      wait until rising_edge(Clk);
   end loop;
 
end procedure;

--Data buffer test
procedure TestDataBuffer(
   variable v_TestData        : in t_1xAT10;
   signal DataIn      : out t_1xAT10;
   signal DataInValid : out std_logic;
   signal NewPopOut   : in std_logic;
   signal DataOut     : in t_1xAT10Array 
) is
  
begin
   wait until rising_edge(Clk);
      DataIn <= v_TestData;
      DataInValid <= '1';
      
      for i in 0 to 11 loop
         wait until rising_edge(Clk);
      end loop;     
   
      DataInValid <= '0';
   
   wait until rising_edge(Clk);
      DataInValid <= '0';
   wait until DataOut(0) = v_TestData;
   
end procedure;




end ComplexPrivateTb;
