--File: ComplexPrivateTb_pck.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: ComplexPrivateTb_pck.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
--<Add assumptions>
--Description: 
--<Add description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

package ComplexPrivateTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   --Global signals
   signal Clk      : std_logic := '0';
   signal Rst_n    : std_logic := '1';
   signal ClkCycle : time      := 10.0 ns;

   --  Todo: Add comment
   type t_Complex is record
      re : signed; --Real part
      im : signed; --Imaginary part
   end record;
  
  type t_IntArray is array (natural range <>, natural range <>) of integer; 
   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

   --Testing a set of multiplications
   procedure TestComplexMul(
        signal A   : out t_Complex16;
        signal B   : out t_Complex10;
        signal Res : in  t_Complex16
        );   

--Testing one multiplication
   procedure TestSingleComplexMul(
      signal A   : in t_Complex16;
      signal B   : in t_Complex10;
      signal Res : in  t_Complex16);

--Testing the vector multiplier unit with 16 and 10 bit inputs
procedure TestVectorMul1610(
   signal Hermit16 : out t_1xAT16;
   signal Vector10 : out t_1xAT10;
   signal Result   : in t_Complex20;
   signal DataValid: out std_logic ;
   signal ResultValid: in std_logic ;
   variable v_TestValues : in t_IntArray
);

--Testing the vector multiplier unit with 16 and 16 bit inputs
procedure TestVectorMul1616(
   signal Hermit16 : out t_1xAT16;
   signal Vector16 : out t_1xAT16;
   signal Result   : in t_Complex20;
   signal DataValid: out std_logic ;
   signal ResultValid: in std_logic ;
   variable v_TestValues : in t_IntArray
);
--for testing
procedure TruncatedIntegerComplexVectorMul(
   variable v_InputVectors : in t_IntArray;
   variable v_ExpResIm : out integer;   
   variable v_ExpResRe : out integer;
   constant c_VectorLengthA : integer;
   constant c_VectorLengthB : integer 
);

--for cost function
procedure HVecMatrixVecMultiplication(
   variable v_InputIntegerMatrix : in t_IntArray;
   variable v_ExpResIm : out integer;
   variable v_ExpResRe : out integer;
   constant c_VectorLength :integer;
   constant c_MatrixLength :integer
);

procedure TestHVecMatrixVecMultiplication( 
   variable v_InputIntegerMatrix : in t_IntArray;
   signal InputMatrix : out t_ATxAT16;
   signal InputVector : out t_1xAT10;
   signal DataValid   : out std_logic;
   signal Result      : in t_Complex20;
   signal ResultValid : in std_logic
);

--CostFunction calculation of expercted result
procedure CostFunction(
   variable v_InputIntegerMatrix: in t_IntArray;
   variable v_ExpResIm : out integer;
   variable v_ExpResRe : out integer;
   constant c_VectorLengthA : integer;
   constant c_VectorLengthB : integer
);

procedure TestCostFunction( 
      variable v_InputIntegerMatrix : in t_IntArray;
      signal InputMatrix : out t_ATxAT16;
      signal InputVectorW : out t_1xAT10;
      signal InputVectorC : out t_1xAT16;
      signal DataValid   : out std_logic;
      signal P1          : out t_Complex20;
      signal Result      : in t_Complex20;
      signal Ready       : in std_logic;
      signal ResultValid : in std_logic 
   );

procedure TestInsertionSort(
   variable v_InputIntegerMatrix : in t_IntArray;
   signal OutputWeights          : in t_1xAT10;
   signal DataValid              : in std_logic;
   signal InputWeights           : out t_1xAT10;
   signal InputCost              : out t_Complex20;
   signal InputCostAddress       : out std_logic_vector(6 downto 0);
   signal InputWeightAddress     : out std_logic_vector(6 downto 0);
   signal NewPopIn               : out std_logic  
);

procedure SortIndividuals(
   variable v_InputIntegerMatrix: in t_IntArray;
   variable v_OutputIntegerMatrix: out t_IntArray
);

--Mutation test procedure
procedure TestMutation(
   signal WeightsOut       : in t_1xAT10;
   signal RandomBits       : out std_logic_vector (1259 downto 0);
   signal WeightsIn        : out t_1xAT10;
   signal MutationRate     : out t_IntegerArray;
   variable v_TestValues   : in t_IntArray;
   variable v_TestRandomBits       : in std_logic_vector (1259 downto 0);
   variable v_MutationRate           : t_IntegerArray
);

--For testing of the breeding module
procedure TestBreeding(
   signal NewPopIn          :out std_logic;
   signal ParrentA          :out t_1xAT10;
   signal ParrentB          :out t_1xAT10;
   signal RandomBits        :out std_logic_vector (179 downto 0);
   signal IndividualOut     : in t_1xAT10;
   signal NewPopOut         : in std_logic;
   variable v_TestValues      : in t_IntArray;
   variable v_TestRandomBits  : std_logic_vector (179 downto 0)
);
--Data buffer test
procedure TestDataBuffer(
   variable v_TestData        : in t_1xAT10;
   signal DataIn      : out t_1xAT10;
   signal DataInValid : out std_logic;
   signal NewPopOut   : in std_logic;
   signal DataOut     : in t_1xAT10Array 
   );


end ComplexPrivateTb;
