--File: CostFunctionTb_sim.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: CostFunctionTb_sim.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
-- <Add Assumptions>
--Description: 
-- <Add Description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

library UTILSLIB;
use UTILSLIB.BasicTypes.all;

library TESTUTILSLIB;
use TESTUTILSLIB.TestbenchFunctions.all;
use TESTUTILSLIB.stdio_h.all;

library TESTUTILSSIM;
--Only use one of theese
use TESTUTILSSIM.GenTbFifo;  --Comment to use TbFifo instead of GenTbFifo
--use TESTUTILSSIM.TbFifo.all; --Unomment to use TbFifo instead of GenTbFifo

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library GENETICALGSIM;
use GENETICALGSIM.ComplexPrivateTb.all;

architecture sim of CostFunctionTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------

   -- DUT signals
   signal Clk      : std_logic := '0';
   signal Rst_n    : std_logic := '1';
   signal ClkCycle : time      := 10.0 ns;
   
   --user signals
   signal InputMatrix    : t_ATxAT16;
   signal InputVectorC   : t_1xAT16;
   signal InputVectorW   : t_1xAT10;
   signal DataValid      : std_logic;
   signal P1             : t_Complex20;
   signal Result         : t_Complex20;
   signal ResultValid    : std_logic;
   signal Ready          : std_logic;
   
   constant c_minValue10  : integer := -512;
   constant c_maxValue10  : integer := 511;
   constant c_minValue16  : integer := -32768;
   constant c_maxValue16  : integer := 32767;

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Comment this section to use the TbFifo instead GenTbFifo
   --    This code requires VHDL 2008
   --Define the record used by tcl
   type t_ElementType is record
      SimCase  : natural range 0 to 255;
      DataSets : natural;
   end record;

   --instansiate the tcl record 
   constant c_ElementTypeInit : t_ElementType := (SimCase  => 0,
                                                  DataSets => 0);

   --Create a Generic fifo used for commands from tcl
   --The input type is t_ElementType
   package GenFifo is new TESTUTILSSIM.GenTbFifo generic map(t_ElementType     => t_ElementType,
                                                             c_ElementTypeInit => c_ElementTypeInit);

   shared variable CmdFifo : GenFifo.t_GenTbFifo;  --Tcl command fifo.
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Uncomment to use TbFifo instead of GenTbFifo
   --    This code requires VHDL 2002
   --   shared variable CmdFifo  : t_TbFifo; --Tcl command fifo
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––



   -- Testbench signals 
   signal tcl_Cmd      : t_ElementType := c_ElementTypeInit;
   signal tcl_MainTrig : std_logic     := '0';  --Trigger the main case
   signal SimComplete  : std_logic     := '0';  --Informs tcl that the case is complete

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Processes
   ----------------------------------------------------------------------------

   p_Clk : Clock(Clk, ClkCycle);

   -- Receive commands from the Tcl scripts.
   p_RxCmd : process is
   begin
      wait until tcl_MainTrig = '1';
      log(ID_LOG_HDR, "Adding SimCase to CmdFifo:" & to_string(tcl_Cmd.SimCase,4));
      CmdFifo.push(tcl_Cmd);
      wait until tcl_MainTrig = '0';
   end process;


   --Main simulation process. Triggered by tcl.
   p_Main : process is
      variable t        : time          := 0 ns;
      variable v_SimCmd : t_ElementType := c_ElementTypeInit;
      variable v_InputIntegerMatrix : t_IntArray(0 to 22,0 to c_AT-1); 
   begin
      
      log(ID_LOG_HDR, "Start of simulation.");

      while true loop
         
         -- Wait for Sim Cmd in FIFO, then read Sim Cmd 
         if CmdFifo.fifo_depth = 0 then
            wait until tcl_MainTrig = '1';
            wait until tcl_MainTrig = '0';
         end if;

         t := now;
         v_SimCmd := CmdFifo.pop;

         --Inform tcl that the case is running
         SimComplete <= '0';

         case v_SimCmd.SimCase is
            --
            -- Random data
            --
            when 0 =>

            for i in 1 to 1000 loop  
            
               for j in 0 to 1 loop
                  for k in 0 to c_AT-1 loop     --complex conj of v_InputIntegerMatrix in integer form means that -511 must be the smallest value
                     v_InputIntegerMatrix(j,k) := random(-c_maxValue10,c_maxValue10); 
                  end loop;
               end loop; 
            
               for j in 2 to 19 loop
                  for k in 0 to c_AT-1 loop
                     v_InputIntegerMatrix(j,k) := random(c_minValue16,c_maxValue16); 
                  end loop;
               end loop; 
               
               for j in 20 to 22 loop
                  for k in 0 to c_AT-1 loop     --complex conj of v_InputIntegerMatrix in integer form means that -511 must be the smallest value
                     v_InputIntegerMatrix(j,k) := random(-c_maxValue16,c_maxValue16); 
                  end loop;
               end loop; 

               wait until rising_edge(Clk) and ready = '1';
               
               TestCostFunction( 
                  v_InputIntegerMatrix    => v_InputIntegerMatrix,
                  InputMatrix             => InputMatrix,
                  InputVectorW            => InputVectorW,
                  InputVectorC            => InputVectorC,
                  DataValid               => DataValid  ,
                  P1                      => P1,
                  Result                  => Result     ,
                  ResultValid             => ResultValid,
                  Ready                   => Ready);
            
            end loop;

            --
            -- Force the reset from the tcl
            --
            when 254 =>
               log(ID_LOG_HDR, "Resetting design.");
               wait until rising_edge(Clk);
               Rst_n <= '0';
               wait for 10*ClkCycle;
               Rst_n <= '1';
               wait until rising_edge(Clk);
            --
            -- Display summary
            --
            when 255 =>
               wait until rising_edge(Clk);
               report_alert_counters(VOID);

               --Inform tcl that the case is compete
               SimComplete <= '1';
            when others =>
         end case;
      end loop;
   end process;

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   i_CostFunction : entity GENETICALGLIB.CostFunction(rtl)
   port map (
   Rst_n => Rst_n,
   Clk   => Clk,
   InputMatrix    => InputMatrix,
   InputVectorC   => InputVectorC,
   InputVectorW   => InputVectorW,
   DataValid      => DataValid,
   P1             => P1,
   Result         => Result,
   ResultValid    => ResultValid,
   Ready          => Ready
   );
   

end sim;
