--File: GeneticAlgTb_sim.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: GeneticAlgTb_sim.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: mathiab not found in /home/mathiab/misc/common/userName
--
--Assumptions: 
--<Add assumptions>
--Description: 
--<Add description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

library UTILSLIB;
use UTILSLIB.BasicTypes.all;

library TESTUTILSLIB;
use TESTUTILSLIB.TestbenchFunctions.all;
use TESTUTILSLIB.stdio_h.all;

library TESTUTILSSIM;
--Only use one of theese
use TESTUTILSSIM.GenTbFifo;  --Comment to use TbFifo instead of GenTbFifo
--use TESTUTILSSIM.TbFifo.all; --Unomment to use TbFifo instead of GenTbFifo

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library GENETICALGSIM;
use GENETICALGSIM.ComplexPrivateTb.all;

library modelsim_lib;
use modelsim_lib.util.all;

architecture sim of GeneticAlgTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------

   -- DUT signals
   signal Correlation       : t_ATxAT16;
   signal AutoCorrelation   : t_1xAT16;
   signal P1                : t_Complex20;
   signal MutationRate      : t_IntegerArray(0 to 3);
   signal BreedRandomBits   : std_logic_vector(179 downto 0);
   signal MutRandomBits     : std_logic_vector (1259 downto 0);
   signal OutputWeights     : t_1xAT10;
   constant c_minValue10  : integer := -512;
   constant c_maxValue10  : integer := 511;
   constant c_minValue16  : integer := -32768;
   constant c_maxValue16  : integer := 32767;
   constant c_minValue20  : integer := -524288;
   constant c_maxValue20  : integer := 524287;
   
   signal SystemCost       : t_Complex20;
   signal SystemWeights    : t_1xAT10;
   signal SystemNewPopOut  : std_logic;
   signal SystemNewPopIn  : std_logic;


   
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Comment this section to use the TbFifo instead GenTbFifo
   --    This code requires VHDL 2008
   --Define the record used by tcl
   type t_ElementType is record
      SimCase  : natural range 0 to 255;
      DataSets : natural;
   end record;

   --instansiate the tcl record 
   constant c_ElementTypeInit : t_ElementType := (SimCase  => 0,
                                                  DataSets => 0);

   --Create a Generic fifo used for commands from tcl
   --The input type is t_ElementType
   package GenFifo is new TESTUTILSSIM.GenTbFifo generic map(t_ElementType     => t_ElementType,
                                                             c_ElementTypeInit => c_ElementTypeInit);

   shared variable CmdFifo : GenFifo.t_GenTbFifo;  --Tcl command fifo.
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Uncomment to use TbFifo instead of GenTbFifo
   --    This code requires VHDL 2002
   --   shared variable CmdFifo  : t_TbFifo; --Tcl command fifo
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––



   -- Testbench signals 
   signal tcl_Cmd      : t_ElementType := c_ElementTypeInit;
   signal tcl_MainTrig : std_logic     := '0';  --Trigger the main case
   signal SimComplete  : std_logic     := '0';  --Informs tcl that the case is complete

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Processes
   ----------------------------------------------------------------------------

   p_Clk : Clock(Clk, ClkCycle);

   -- Receive commands from the Tcl scripts.
   p_RxCmd : process is
   begin
      wait until tcl_MainTrig = '1';
      log(ID_LOG_HDR, "Adding SimCase to CmdFifo:" & to_string(tcl_Cmd.SimCase,4));
      CmdFifo.push(tcl_Cmd);
      wait until tcl_MainTrig = '0';
   end process;


   --Main simulation process. Triggered by tcl.
   p_Main : process is
      variable t        : time          := 0 ns;
      variable v_SimCmd : t_ElementType := c_ElementTypeInit;
   begin
      -- Allow simulation to continue regardless of errors
      set_alert_stop_limit(ERROR, 10);   

      log(ID_LOG_HDR, "Start of simulation.");

      while true loop
         
         -- Wait for Sim Cmd in FIFO, then read Sim Cmd 
         if CmdFifo.fifo_depth = 0 then
            wait until tcl_MainTrig = '1';
            wait until tcl_MainTrig = '0';
         end if;

         t := now;
         v_SimCmd := CmdFifo.pop;

         --Inform tcl that the case is running
         SimComplete <= '0';

         case v_SimCmd.SimCase is
            --
            -- Random data
            --
            when 0 =>
            SystemNewPopIn <= '0';
            --signal_force("/i_GeneticAlg/i_DelayDataBuffer/NewPopOut","1",0 ns,freeze);
            for i in 0 to 1 loop
               wait until rising_edge(Clk);

               for rad in 0 to c_AT-1 loop
                  AutoCorrelation(rad).re <= to_signed(random(c_minValue16,c_maxValue16),AutoCorrelation(0).re'length);           
                  AutoCorrelation(rad).im <= to_signed(random(c_minValue16,c_maxValue16),AutoCorrelation(0).re'length);           
                  for col in 0 to c_AT-1 loop
                     Correlation(rad)(col).re <= to_signed(random(c_minValue16,c_maxValue16),Correlation(0)(0).re'length);
                     Correlation(rad)(col).im <= to_signed(random(c_minValue16,c_maxValue16),Correlation(0)(0).re'length);
                  end loop;
               end loop;

               P1.re <= to_signed(random(c_minValue20,c_maxValue20),P1.re'length);
               P1.im <= to_signed(random(c_minValue20,c_maxValue20),P1.re'length);
                        
               for k in 0 to 3 loop
                  MutationRate(k) <= 7-k;
               end loop;
               if i = 1 then
                  SystemNewPopIn <= '1';
               end if;

               for k in 0 to 10000 loop      --100 iterations per sample
                  BreedRandomBits   <= random(180);
                  MutRandomBits     <= random(1260);
                  wait until rising_edge(Clk);
                  if k = 200 then
                     signal_release("/i_GeneticAlg/i_DelayDataBuffer/ReadData");
                     --signal_release("/i_GeneticAlg/i_DelayDataBuffer/NewPopOut");
                  end if;
               end loop;

            end loop;


            --
            -- Force the reset from the tcl
            --
            when 254 =>
               --Forcing buffer data
               signal_force("/i_GeneticAlg/i_DelayDataBuffer/ReadData","16#0", 0 ns, freeze);
               
               log(ID_LOG_HDR, "Resetting design.");
               wait until rising_edge(Clk);
               Rst_n <= '0';
               wait for 10*ClkCycle;
               Rst_n <= '1';
               wait until rising_edge(Clk);
            --
            -- Display summary
            --
            when 255 =>
               wait until rising_edge(Clk);
               report_alert_counters(VOID);

               --Inform tcl that the case is compete
               SimComplete <= '1';
            when others =>
         end case;
      end loop;
   end process;

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   
   i_GeneticAlg : entity GENETICALGLIB.GeneticAlg(str)
      port map (
      Rst_n             => Rst_n,
      Clk               => Clk,
      Correlation       => Correlation,
      AutoCorrelation   => AutoCorrelation,
      P1                => P1,
      MutationRate      => MutationRate,
      BreedRandomBits   => BreedRandomBits,
      MutRandomBits     => MutRandomBits,
      OutputWeights     => OutputWeights
   );


   i_SerialCostFunction : entity GENETICALGLIB.SerialCostFunction(rtl)
      port map (
      Rst_n          => Rst_n,
      Clk            => Clk,
      InputMatrix    => Correlation,
      InputVectorC   => AutoCorrelation,
      InputVectorW   => OutputWeights,
      P1             => P1,
      NewPopIn       => systemNewPopIn,
      Result         => SystemCost,
      NewPopOut      => SystemNewPopOut,
      OutputCostAddress => open,
      OutputWeightAddress => open
   ); 
   



end sim;
