--File: InsertionSortTb_sim.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: InsertionSortTb_sim.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
-- <Add Assumptions>
--Description: 
-- <Add Description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

library UTILSLIB;
use UTILSLIB.BasicTypes.all;

library TESTUTILSLIB;
use TESTUTILSLIB.TestbenchFunctions.all;
use TESTUTILSLIB.stdio_h.all;

library TESTUTILSSIM;
--Only use one of theese
use TESTUTILSSIM.GenTbFifo;  --Comment to use TbFifo instead of GenTbFifo
--use TESTUTILSSIM.TbFifo.all; --Unomment to use TbFifo instead of GenTbFifo

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library GENETICALGSIM;
use GENETICALGSIM.ComplexPrivateTb.all;

architecture sim of InsertionSortTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------

   -- DUT signals
   signal InputCostAddress     : std_logic_vector(6 downto 0);
   signal InputWeightAddress   : std_logic_vector(6 downto 0);

   signal InputWeights   :  t_1xAT10;
   signal InputCost      :  t_Complex20;
   signal NewPopIn       :  std_logic;
   signal OutputWeights  : t_1xAT10;
   signal DataValid      : std_logic;
   constant c_minValue10  : integer := -512;
   constant c_maxValue10  : integer := 511;
   constant c_minValue20  : integer := -524288;
   constant c_maxValue20  : integer := 524287;
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Comment this section to use the TbFifo instead GenTbFifo
   --    This code requires VHDL 2008
   --Define the record used by tcl
   type t_ElementType is record
      SimCase  : natural range 0 to 255;
      DataSets : natural;
   end record;

   --instansiate the tcl record 
   constant c_ElementTypeInit : t_ElementType := (SimCase  => 0,
                                                  DataSets => 0);

   --Create a Generic fifo used for commands from tcl
   --The input type is t_ElementType
   package GenFifo is new TESTUTILSSIM.GenTbFifo generic map(t_ElementType     => t_ElementType,
                                                             c_ElementTypeInit => c_ElementTypeInit);

   shared variable CmdFifo : GenFifo.t_GenTbFifo;  --Tcl command fifo.
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Uncomment to use TbFifo instead of GenTbFifo
   --    This code requires VHDL 2002
   --   shared variable CmdFifo  : t_TbFifo; --Tcl command fifo
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––



   -- Testbench signals 
   signal tcl_Cmd      : t_ElementType := c_ElementTypeInit;
   signal tcl_MainTrig : std_logic     := '0';  --Trigger the main case
   signal SimComplete  : std_logic     := '0';  --Informs tcl that the case is complete

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Processes
   ----------------------------------------------------------------------------

   p_Clk : Clock(Clk, ClkCycle);

   -- Receive commands from the Tcl scripts.
   p_RxCmd : process is
   begin
      wait until tcl_MainTrig = '1';
      log(ID_LOG_HDR, "Adding SimCase to CmdFifo:" & to_string(tcl_Cmd.SimCase,4));
      CmdFifo.push(tcl_Cmd);
      wait until tcl_MainTrig = '0';
   end process;


   --Main simulation process. Triggered by tcl.
   p_Main : process is
      variable t        : time          := 0 ns;
      variable v_SimCmd : t_ElementType := c_ElementTypeInit;
      variable v_InputIntegerMatrix : t_IntArray(0 to 47,0 to c_AT);   
   begin
      
      log(ID_LOG_HDR, "Start of simulation.");

      while true loop
         
         -- Wait for Sim Cmd in FIFO, then read Sim Cmd 
         if CmdFifo.fifo_depth = 0 then
            wait until tcl_MainTrig = '1';
            wait until tcl_MainTrig = '0';
         end if;

         t := now;
         v_SimCmd := CmdFifo.pop;

         --Inform tcl that the case is running
         SimComplete <= '0';

         case v_SimCmd.SimCase is
            --
            -- Testing with random data
            --
            when 0 =>
           
               for i in 1 to 2 loop  
                
                  for j in 0 to 47 loop
                     for k in 0 to c_AT-1 loop
                        if i = 1 then
                           v_InputIntegerMatrix(j,k) := j; 
                        else
                           v_InputIntegerMatrix(j,k) := random(c_minValue10,c_maxValue10); 
                        end if;
                     end loop;
                     if i = 1 then
                        v_InputIntegerMatrix(j,c_AT) := j;
                     else
                        v_InputIntegerMatrix(j,c_AT) := random(c_minValue20,c_maxValue20);
                     end if;
                  end loop;
                   
                  wait until rising_edge(Clk);
            
                  TestInsertionSort( 
                     v_InputIntegerMatrix => v_InputIntegerMatrix,
                     InputWeights         => InputWeights,
                     InputCost            => InputCost,
                     NewPopIn             => NewPopIn,
                     OutputWeights        => OutputWeights,
                     InputCostAddress     => InputCostAddress,
                     InputWeightAddress   => InputWeightAddress,
                     DataValid            => DataValid
                     );
               end loop;
            --
            -- Force the reset from the tcl
            --
            when 254 =>
               log(ID_LOG_HDR, "Resetting design.");
               wait until rising_edge(Clk);
               Rst_n <= '0';
               wait for 10*ClkCycle;
               Rst_n <= '1';
               wait until rising_edge(Clk);
            --
            -- Display summary
            --
            when 255 =>
               wait until rising_edge(Clk);
               report_alert_counters(VOID);

               --Inform tcl that the case is compete
               SimComplete <= '1';
            when others =>
         end case;
      end loop;
   end process;

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   i_InsertionSort : entity GENETICALGLIB.InsertionSort(rtl)
      port map (
      Rst_n                => Rst_n,
      Clk                  => Clk,
      InputWeights         => InputWeights,
      InputCost            => InputCost,
      InputCostAddress     => InputCostAddress,
      NewPopIn             => NewPopIn,
      InputWeightAddress   => InputWeightAddress,
      OutputWeights        => OutputWeights,
      DataValid            => DataValid
   );
   

end sim;
