--File: MutationTb_sim.vhd (vhdl)
--<Short Description>
--(c) Kongsberg Defence & Aerospace 2015
--$Id: MutationTb_sim.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
-- <Add Assumptions>
--Description: 
-- <Add Description>

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

library BITVISUTILLIB;
use BITVISUTILLIB.types_pkg.all;
use BITVISUTILLIB.string_methods_pkg.all;
use BITVISUTILLIB.adaptations_pkg.all;
use BITVISUTILLIB.methods_pkg.all;
use BITVISUTILLIB.bfm_common_pkg.all;

library UTILSLIB;
use UTILSLIB.BasicTypes.all;

library TESTUTILSLIB;
use TESTUTILSLIB.TestbenchFunctions.all;
use TESTUTILSLIB.stdio_h.all;

library TESTUTILSSIM;
--Only use one of theese
use TESTUTILSSIM.GenTbFifo;  --Comment to use TbFifo instead of GenTbFifo
--use TESTUTILSSIM.TbFifo.all; --Unomment to use TbFifo instead of GenTbFifo

library GENETICALGLIB;
use GENETICALGLIB.GeneticAlgPrivate.all;

library GENETICALGSIM;
use GENETICALGSIM.ComplexPrivateTb.all;

architecture sim of MutationTb is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------

   -- DUT signals
   constant c_minValue10  : integer := -512;
   constant c_maxValue10  : integer := 511;
   signal WeightsIn      : t_1xAT10;
   signal RandomBits     : std_logic_vector(1259 downto 0);
   signal MutationRate   : t_IntegerArray(0 to 3);
   signal WeightsOut     : t_1xAT10;
   signal NewPopIn       : std_logic;
   signal NewPopOut      : std_logic;
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Comment this section to use the TbFifo instead GenTbFifo
   --    This code requires VHDL 2008
   --Define the record used by tcl
   type t_ElementType is record
      SimCase  : natural range 0 to 255;
      DataSets : natural;
   end record;

   --instansiate the tcl record 
   constant c_ElementTypeInit : t_ElementType := (SimCase  => 0,
                                                  DataSets => 0);

   --Create a Generic fifo used for commands from tcl
   --The input type is t_ElementType
   package GenFifo is new TESTUTILSSIM.GenTbFifo generic map(t_ElementType     => t_ElementType,
                                                             c_ElementTypeInit => c_ElementTypeInit);

   shared variable CmdFifo : GenFifo.t_GenTbFifo;  --Tcl command fifo.
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––

   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––
   --    Uncomment to use TbFifo instead of GenTbFifo
   --    This code requires VHDL 2002
   --   shared variable CmdFifo  : t_TbFifo; --Tcl command fifo
   --––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––––



   -- Testbench signals 
   signal tcl_Cmd      : t_ElementType := c_ElementTypeInit;
   signal tcl_MainTrig : std_logic     := '0';  --Trigger the main case
   signal SimComplete  : std_logic     := '0';  --Informs tcl that the case is complete

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Processes
   ----------------------------------------------------------------------------

   p_Clk : Clock(Clk, ClkCycle);

   -- Receive commands from the Tcl scripts.
   p_RxCmd : process is
   begin
      wait until tcl_MainTrig = '1';
      log(ID_LOG_HDR, "Adding SimCase to CmdFifo:" & to_string(tcl_Cmd.SimCase,4));
      CmdFifo.push(tcl_Cmd);
      wait until tcl_MainTrig = '0';
   end process;


   --Main simulation process. Triggered by tcl.
   p_Main : process is
      variable t        : time          := 0 ns;
      variable v_SimCmd : t_ElementType := c_ElementTypeInit;
      variable v_TestValues : t_IntArray(0 to 1,0 to 8);
      variable v_TestRandomBits : std_logic_vector (1259 downto 0);
      variable v_MutationRate : t_IntegerArray(0 to 3);
   begin
      
      log(ID_LOG_HDR, "Start of simulation.");

      while true loop
         
         -- Wait for Sim Cmd in FIFO, then read Sim Cmd 
         if CmdFifo.fifo_depth = 0 then
            wait until tcl_MainTrig = '1';
            wait until tcl_MainTrig = '0';
         end if;

         t := now;
         v_SimCmd := CmdFifo.pop;

         --Inform tcl that the case is running
         SimComplete <= '0';

         case v_SimCmd.SimCase is
            --
            -- Testing with random data
            --
            when 0 =>
            
            wait until rising_edge(Clk);
            NewPopIn <= '0'; 
            wait until rising_edge(Clk);
            NewPopIn <= '1';
            wait until rising_edge(Clk);
            NewPopIn <= '0'; 
            
            for i in 0 to 1000 loop
               v_TestRandomBits := random(1260);
               
               for j in 0 to 1 loop
                  for k in 0 to 8 loop
                     v_TestValues(j,k) := random(c_minValue10,c_maxValue10); 
                  end loop;
               end loop;
               for k in 0 to 3 loop
                  v_MutationRate(k) := random(4,7);
               end loop;
               wait until rising_edge(Clk);
            
               TestMutation(
                  WeightsOut       => WeightsOut,       
                  RandomBits       => RandomBits,    
                  WeightsIn        => WeightsIn,      
                  MutationRate     => MutationRate, 
                  v_TestValues     => v_TestValues,   
                  v_TestRandomBits => v_TestRandomBits,
                  v_MutationRate   => v_MutationRate);
            
            end loop;
            
            --
            -- Force the reset from the tcl
            --
            when 254 =>
               log(ID_LOG_HDR, "Resetting design.");
               wait until rising_edge(Clk);
               Rst_n <= '0';
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               wait until rising_edge(Clk);
               Rst_n <= '1';
               wait until rising_edge(Clk);
            --
            -- Display summary
            --
            when 255 =>
               wait until rising_edge(Clk);
               report_alert_counters(VOID);

               --Inform tcl that the case is compete
               SimComplete <= '1';
            when others =>
         end case;
      end loop;
   end process;

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   i_Mutation : entity GENETICALGLIB.Mutation(rtl)
   port map (
      Rst_n          => Rst_n,
      Clk            => Clk,
      WeightsIn      => WeightsIn,
      RandomBits     => RandomBits,
      MutationRate   => MutationRate,
      NewPopIn       => NewPopIn,
      WeightsOut     => WeightsOut,
      NewPopOut      => NewPopOut
   );
   

end sim;
