--File: Breeding_rtl.vhd (vhdl)
--Module used for breeding in genetic algorithm
--(c) Kongsberg Defence & Aerospace 2015
--$Id: Breeding_rtl.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
-- <Add Assumptions>
--Description: 
-- The breeding module takes two "parrents" of type t_1xAT10. The IndividualOut circulates between beeing
-- one parrent, the other parent, one child and a second child for each clock cycle. The children are made
-- by using the RandomBits to chose which of the parrents each of the bits should be taken from. The second
-- child has bits picked from the oposite parrent of the first child. NewPopIn synchronises the output such
-- that NewPopOut reflects a high NewPopIn and at the same time the IndividualOut is ParrentA.


architecture rtl of Breeding is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   signal Counter : integer range 0 to 3;

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   

   ----------------------------------------------------------------------------
   -- Clocked processes
   ----------------------------------------------------------------------------
   
   p_MyProcess : process(Clk) is
      variable NewPopOutWaiting : std_logic;
   begin
      if rising_edge(Clk) then
         if Rst_n = '0' then
            Counter <= 0;
            IndividualOut <= (others => (others => (others => '0')));
            NewPopOut <= '0';
            NewPopOutWaiting := '0';
         else
            NewPopOut <= '0';
            -- Counter is set to 3 to syncronise the output. This way for every set of parrents the order
               -- will be ParrentA, ParrentB, two children.
            if NewPopIn = '1' then
               Counter <= 3;
               NewPopOutWaiting := '1';
            else
               --Counter controls if ParrentA, ParrentB or which of the children that is transmitted
               if Counter < 3 then
                  Counter <= Counter +1;
               else
                  Counter <= 0;
               end if;
            end if;
           
            --output selection based on counter
            
            --Child 1
            if Counter = 0 then
               for weightNr in 0 to c_AT-1 loop
                  for bitNr in c_NumBitsLow -1 downto 0 loop
                     --random bits are used to select which parrent the bits of the children should come from.
                     if RandomBits((weightNr*c_NumBitsLow + bitNr)*2) = '1' then   
                        IndividualOut(weightNr).re(bitNr) <= ParrentA(weightNr).re(bitNr);
                     else
                        IndividualOut(weightNr).re(bitNr) <= ParrentB(weightNr).re(bitNr);   
                     end if;
                    
                     if RandomBits((weightNr*c_NumBitsLow + bitNr)*2+1) = '1' then   
                        IndividualOut(weightNr).im(bitNr) <= ParrentA(weightNr).im(bitNr);
                     else
                        IndividualOut(weightNr).im(bitNr) <= ParrentB(weightNr).im(bitNr);   
                     end if;
                  end loop;
               end loop;
            --Child 2
            elsif Counter = 1 then
               for weightNr in 0 to c_AT-1 loop
                  for bitNr in c_NumBitsLow -1 downto 0 loop
                     if RandomBits((weightNr*c_NumBitsLow + bitNr)*2) = '0' then   
                        IndividualOut(weightNr).re(bitNr) <= ParrentA(weightNr).re(bitNr);
                     else
                        IndividualOut(weightNr).re(bitNr) <= ParrentB(weightNr).re(bitNr);   
                     end if;
                    
                     if RandomBits((weightNr*c_NumBitsLow + bitNr)*2+1) = '0' then   
                        IndividualOut(weightNr).im(bitNr) <= ParrentA(weightNr).im(bitNr);
                     else
                        IndividualOut(weightNr).im(bitNr) <= ParrentB(weightNr).im(bitNr);   
                     end if;
                  end loop;
               end loop;
            --ParrentA
            elsif Counter = 2 then
               IndividualOut <= ParrentA;
               --NewPopOut is only sent when ParrentA is sent
               if NewPopOutWaiting = '1' then
                  NewPopOut <= '1';
                  NewPopOutWaiting := '0';
               end if;
            --ParrentB
            else
               IndividualOut <= ParrentB;
            end if;
         end if; 
      end if;
   end process; 
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------

end rtl;

