--File: ComplexMul_rtl.vhd (vhdl)
--16 bit by 10 bit complex multiplication
--(c) Kongsberg Defence & Aerospace 2015
--$Id: ComplexMul16b10b_rtl.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
--<Add assumptions>
--Description: Complex multiplication of 16 bit complex number A with 10 bit complex number B
--Latency of 3 ClockCycles


architecture rtl of ComplexMul16b10b is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------


   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   

   ----------------------------------------------------------------------------
   -- Clocked processes
   ----------------------------------------------------------------------------
   
   p_ComplexMul : process(Clk) is
  
   --variables to hold partial result
   variable v_AreBre : signed(A.re'length+B.re'length-1 downto 0);
   variable v_AreBim : signed(A.re'length+B.im'length-1 downto 0);
   variable v_AimBre : signed(A.im'length+B.re'length-1 downto 0);
   variable v_AimBim : signed(A.im'length+B.im'length-1 downto 0);
   --variables to hold result (output register)
   variable v_ResRe : signed(A.im'length+B.im'length-1 downto 0);
   variable v_ResIm : signed(A.im'length+B.im'length-1 downto 0);
  
   begin
      if rising_edge(Clk) then
         if Rst_n = '0' then
            Res.re <= (others=>'0');  
            Res.im <= (others=>'0');
            v_AreBre := (others=>'0');
            v_AreBim := (others=>'0');
            v_AimBre := (others=>'0');
            v_AimBim := (others=>'0');
            v_ResRe := (others=>'0');
            v_ResIm := (others=>'0');
              
         else
            --output register is coupled to output
            Res.re <= v_ResRe(A.im'length+B.im'length-1 downto B.im'length);
            Res.im <= v_ResIm(A.im'length+B.im'length-1 downto B.im'length);
            
            --output register value is calculated from combining partial results
            v_ResRe := v_AreBre - v_AimBim;
            v_ResIm := v_AimBre + v_AreBim;
            
            --partial results are found using one clock cyle multiplication.
            v_AreBre := A.re * B.re;
            v_AreBim := A.re * B.im;
            v_AimBre := A.im * B.re;
            v_AimBim := A.im * B.im;
         
         end if;
      end if;
   end process; 
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------

end rtl;

