--File: ComplexVectorHermitianMul_rtl.vhd (vhdl)
--Fast 16 by 10 bit complex vector mulitplier
--(c) Kongsberg Defence & Aerospace 2015
--$Id: ComplexVectorMul1610_str.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Assumptions: 
--<Add assumptions>
--Description: 
-- The ComplexVectorMul1610 perform vector multiplication between the 16 bit signed complex
-- vector Hermitian16 and the 10 bit signed complex vector Vector 10. The delay depends on the
-- length of the vectors, but a tree adder and as many multipliers as there is elements is used,
-- so this is a fast resourses which uses a lot of resources. This vector multiplier is also fully
-- pipelined so that it can handle one vector multiplication per cycle.(throughput) 


architecture str of ComplexVectorMul1610 is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   signal Addends     : t_1xN16(0 to c_AT-1); --results from all element multiplications
   constant c_numAddends : natural := c_AT; --number of results from element muliplications to be added
   signal MultiplierDelayDataValid : std_logic_vector (2 downto 0); --used give DataValid to adder when
                                                                        --data has finished through mulitplier.
   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   -- Delaying Datavalid While waiting for multiplier
   p_DelayDataValid : process (Clk) is
   begin
      if rising_edge(Clk) then
         if (Rst_n = '0') then
            MultiplierDelayDataValid <= (others => '0') ;
         else
            MultiplierDelayDataValid <= MultiplierDelayDataValid(1 downto 0) & Datavalid;
         end if;
      end if;
   end process p_DelayDataValid;
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
--generating multipliers
generatingMultipliers: for i in 0 to (c_AT-1) generate
   i_ComplexMul16b10b : entity GENETICALGLIB.ComplexMul16b10b(rtl)
   port map (
   Rst_n => Rst_n,
   Clk   => Clk,
   A.re  => Hermit16(i).re,
   A.im  => Hermit16(i).im,
   B     => Vector10(i),
   Res   => Addends(i)
   );
end generate;


i_ComplexAdderTree : entity GENETICALGLIB.ComplexAdderTree(rtl)
   generic map (
      c_NumAddends => c_NumAddends
   )
   port map (
   Rst_n       => Rst_n,
   Clk         => Clk,
   Addends     => Addends,
   Result      => Result,
   DataValid   => MultiplierDelayDataValid(2),
   ResultValid => ResultValid
);


end str;

