--File: GeneticAlg_pck.vhd (vhdl)
--package which holds the common constants, types and functions of the genetic algorithm module
--(c) Kongsberg Defence & Aerospace 2015
--$Id: GeneticAlgPrivate_pck.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Description: 
--package which holds the common constants, types and functions of the genetic algorithm module

library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;

package GeneticAlgPrivate is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
type t_Complex16 is record
   re : signed(15 downto 0); --real part
   im : signed(15 downto 0); --imaginary part
end record;

type t_Complex10 is record
   re : signed(9 downto 0); --real part
   im : signed(9 downto 0); --imaginary part
end record;

type t_Complex20 is record
   re : signed(19 downto 0); --real part
   im : signed(19 downto 0); --imaginary part
end record;

constant c_NumTaps      : natural := 3; --number of taps per non reference antenna
constant c_NumAnt       : natural := 3; --number of non reference antennas
constant c_AT           : natural := c_NumAnt*c_NumTaps; --total number of non reference taps
constant c_NumBitsSum   : natural := 20; --number of bits in the result of the arithmetic operations
constant c_NumBitsHigh  : natural := 16; --number of bits in the high resolution inputs
constant c_NumBitsLow   : natural := 10; --number of bits in the low resolution inputs
constant c_MulLatency   : natural := 3;  --Latency through fully pipelined complex multiplier
constant c_PopSize      : natural := 24; --population size
constant c_NumOfPop     : natural := 4;  --number of populations

type t_MxN20 is array (natural range <> , natural range <>) of t_Complex20; --m x n complex 20 bit array
type t_1xN20 is array (natural range <>) of t_Complex20; --1 x n complex 20 bit vector
type t_1xN16 is array (natural range <>) of t_Complex16; --1 x n complex 16 bit vector
type t_1xAT16 is array (0 to c_AT-1) of t_Complex16;     --1 x c_AT complex 16 bit vector
type t_1xAT10 is array (0 to c_AT-1) of t_Complex10;     --1 x c_AT complex 10 bit vector
type t_ATxAT16 is array (0 to c_AT-1) of t_1xAT16;       --c_AT x c_AT complex 16 bit array
type t_1xAT10Array is array(0 to 1) of t_1xAT10;         --size 2 vector of 1 x c_AT complex 10 bit vector
type t_1xAT10ArrayUnCon is array (natural range <>) of t_1xAT10; --unconstrained size vector of 1 x c_AT complex 10 bit vector
type t_IntegerArray is array( natural range <>) of integer range 4 to 7; --unconstrained sized array of integers range 4 to 7

--logarithmic function with integer result rounded down
function logFloor(Base : positive;
             X    : positive) return natural;
--logarithmic function with integer result rounded up
function logCeil(Base : positive;
             X    : positive) return natural;

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

end GeneticAlgPrivate;
