--File: GeneticAlg_str.vhd (vhdl)
--Top file of the gentic algorithm module
--(c) Kongsberg Defence & Aerospace 2015
--$Id: GeneticAlg_str.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: mathiab not found in /home/mathiab/misc/common/userName
--
--Description: 
--All the sub modules of the genetic algorithm is coupled together. There are placed delay registers
--to reach timing goals in PAR.


architecture str of GeneticAlg is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   constant c_DelayLength        : integer := 1;
   signal DelayWeights           : t_1xAT10ArrayUnCon(0 to c_DelayLength - 1);
   signal DelayNewPop            : std_logic_vector (0 to c_DelayLength -1);

   signal Cost2SortNewPop        : std_logic;
   signal Cost2SortCost          : t_Complex20;
   signal Cost2SortCostAddress   : std_logic_vector(6 downto 0);
   signal Cost2SortWeightAddress : std_logic_vector(6 downto 0);
   signal Buff2BreedWeightSet    : t_1xAT10Array; 
   signal Buff2BreedNewPop       : std_logic;
   signal Sort2BuffDataValid     : std_logic;
   signal Sort2BuffWeights       : t_1xAT10; 
   signal Breed2MutNewPop        : std_logic;
   signal Breed2MutWeights       : t_1xAT10;
   signal Mut2CostNewPop        : std_logic;
   signal Mut2CostWeights       : t_1xAT10;

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin
   
   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   -- Make signals go through register to simplify routing in terms of reaching timing
   p_DelayingSignals : process (Clk) is
   begin
      if rising_edge(Clk) then
         if (Rst_n = '0') then
            DelayNewPop <= (others => '0');
            DelayWeights <= (others => (others => (others => (others => '0')))); 
         else
            for i in 1 to c_DelayLength - 1 loop
               DelayNewPop(i)  <= DelayNewPop(i-1);
               DelayWeights(i) <= DelayWeights(i-1);
            end loop;
               DelayWeights(0) <= Mut2CostWeights;
               DelayNewPop(0)  <= Mut2CostNewPop;
         end if;
      end if;
   end process p_DelayingSignals;
   

   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   
   -- Cost function module 1 individual in and 1 individual out per cycle
   i_SerialCostFunction : entity GENETICALGLIB.SerialCostFunction(rtl)
      port map (
      Rst_n                => Rst_n,
      Clk                  => Clk,
      InputMatrix          => Correlation,
      InputVectorC         => AutoCorrelation,
      InputVectorW         => DelayWeights(c_DelayLength - 1),
      P1                   => P1,
      NewPopIn             => DelayNewPop(c_DelayLength - 1),
      Result               => Cost2SortCost,
      NewPopOut            => Cost2SortNewPop,
      OutputCostAddress    => Cost2SortCostAddress,
      OutputWeightAddress  => Cost2SortWeightAddress
   ); 

   --Insertion sort module
   i_InsertionSort : entity GENETICALGLIB.InsertionSort(rtl)
   port map (
      Rst_n             => Rst_n,
      Clk               => Clk,
      InputWeights      => DelayWeights(c_DelayLength - 1),
      InputCost         => Cost2SortCost,
      InputCostAddress  => Cost2SortCostAddress,
      NewPopIn          => Cost2SortNewPop,
      InputWeightAddress=> Cost2SortWeightAddress,
      OutputWeights     => Sort2BuffWeights,
      DataValid         => Sort2BuffDataValid
   );
   
   -- Delay buffer
   i_DelayDataBuffer : entity GENETICALGLIB.DelayDataBuffer(rtl)
      port map (
      Rst_n       => Rst_n,
      Clk         => Clk,
      DataIn      => Sort2BuffWeights,
      DataInValid => Sort2BuffDataValid,
      NewPopOut   => Buff2BreedNewPop,
      DataOut     => Buff2BreedWeightSet,
      CurrWeights => OutputWeights
   );
   
   -- Breeding module
   i_Breeding : entity GENETICALGLIB.Breeding(rtl)
      port map (
      Rst_n          => Rst_n,
      Clk            => Clk,
      NewPopIn       => Buff2BreedNewPop,
      ParrentA       => Buff2BreedWeightSet(0),
      ParrentB       => Buff2BreedWeightSet(1),
      RandomBits     => BreedRandomBits,
      IndividualOut  => Breed2MutWeights,
      NewPopOut      => Breed2MutNewPop
   );
   
   -- Mutation modual
   i_Mutation : entity GENETICALGLIB.Mutation(rtl)
      port map (
      Rst_n          => Rst_n,
      Clk            => Clk,
      WeightsIn      => Breed2MutWeights,
      RandomBits     => MutRandomBits,
      MutationRate   => MutationRate,
      NewPopIn       => Breed2MutNewPop,
      WeightsOut     => Mut2CostWeights,
      NewPopOut      => Mut2CostNewPop
   );
   end str;

