--File: GeneticWrap_rtl.vhd (vhdl)
--wrapper for the GeneticAlg module used for PAR
--(c) Kongsberg Defence & Aerospace 2015
--$Id: GeneticWrap_rtl.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Description: 
--To be able to do PAR on a separat a wrapper is made, the wrapper uses a large shift register to feed the output
--of the output of the module to the input of the module to avoid that the cicuit is simplified.


architecture rtl of GeneticWrap is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   --lagre shift register used to feed the inputs of the module with the outputs
   signal MegaShiftReg : std_logic_vector(4359 downto 0);

   
   signal Correlation       : t_ATxAT16;
   signal AutoCorrelation   : t_1xAT16;
   signal MutRandomBits     : std_logic_vector (1259 downto 0);
   signal OutputWeights     : t_1xAT10;
   signal BreedRandomBits   : std_logic_vector(179 downto 0);
   signal P1                : t_Complex20;
   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin
      
   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   
   --Connect Correlation to shiftreg with output values
   Connections: for rad in 0 to c_AT-1 generate
      Connections2: for col in 0 to c_AT-1 generate
         Correlation(rad)(col).re <= signed(MegaShiftReg(c_NumBitsHigh-1+2*c_NumBitsHigh*col+rad*c_NumBitsHigh*c_AT*2 downto 2*c_NumBitsHigh*col+rad*c_NumBitsHigh*c_AT*2));
         Correlation(rad)(col).im <= signed(MegaShiftReg(c_NumBitsHigh-1+c_NumBitsHigh+2*c_NumBitsHigh*col+rad*c_NumBitsHigh*c_AT*2 downto c_NumBitsHigh+2*c_NumBitsHigh*col+rad*c_NumBitsHigh*c_AT*2));
      end generate;
   end generate; 
   
   --Connect AutoCorrelation to shiftreg with output values
   Connections3: for rad in 0 to c_AT-1 generate
      AutoCorrelation(rad).re <= signed(MegaShiftReg(c_NumBitsHigh-1+c_AT*c_AT*c_NumBitsHigh*2 downto c_AT*c_AT*c_NumBitsHigh*2)); 
      AutoCorrelation(rad).im <= signed(MegaShiftReg(c_NumBitsHigh-1+c_NumBitsHigh+c_AT*c_AT*c_NumBitsHigh*2 downto c_NumBitsHigh+c_AT*c_AT*c_NumBitsHigh*2)); 
   end generate;

   --Connect MutRandomBits to shiftreg with output values
   MutRandomBits <= MegaShiftReg(1259+(c_AT+1)*c_AT*c_NumBitsHigh*2 downto (c_AT+1)*c_AT*c_NumBitsHigh*2); 
   OutWeightBit <= OutputWeights(0).re(0);
      
   --Connect BreedRandomBits to shiftreg with output values
   BreedRandomBits <= MegaShiftReg(4319 downto 4140);

   --Connect P1 to shiftreg with output values
   P1.re <= signed(MegaShiftReg(4339 downto 4320));
   P1.im <= signed(MegaShiftReg(4359 downto 4340));
      
   ----------------------------------------------------------------------------
   -- Clocked processes
   ----------------------------------------------------------------------------
   
   p_MyProcess : process(Clk) is
   variable v_Counter : integer range 0 to c_AT-1;
   variable v_Counter2: integer range 0 to c_NumBitsLow-1;
   begin
      if rising_edge(Clk) then
         if Rst_n = '0' then
            v_Counter2 := 0;
            v_Counter  := 0;
         else
            --shift into the shift register one of the bits on from the output weights 
            MegaShiftReg(0) <= std_logic(OutputWeights(v_Counter).re(v_Counter2));
            --shift the rest of the register
            for i in 1 to 4359 loop
               MegaShiftReg(i) <= MegaShiftReg((i-1));
            end loop;
            
            --counters used to pick one bit from the output
            if v_Counter2 = c_NumBitsLow-1 then
               v_Counter2 := 0;
               if v_Counter = c_AT-1 then
                  v_Counter := 0;
               else
                  v_Counter := v_Counter +1;
               end if;
            else
               v_Counter2 := v_Counter2 + 1;
            end if;
         end if; 
      end if;
   end process; 
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   --the genetic algorithm module
   i_GeneticAlg : entity GENETICALGLIB.GeneticAlg(str)
      port map (
      Rst_n             => Rst_n,
      Clk               => Clk,
      Correlation       => Correlation,
      AutoCorrelation   => AutoCorrelation,
      P1                => P1,
      MutationRate      => MutationRate,
      BreedRandomBits   => BreedRandomBits,
      MutRandomBits     => MutRandomBits,
      OutputWeights     => OutputWeights
   );
end rtl;

