--File: InsertionSort_rtl.vhd (vhdl)
--top level sorting module. sorts n elements in n clock cycles and transmitts the n/2 best of the elements.
--(c) Kongsberg Defence & Aerospace 2015
--$Id: InsertionSort_rtl.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Description: 
--The InsertionSort module takes one element and its cost per cycle and sorts this element according to its 
--cost. The module only transmits the best half of the elemenets. The current sorted elements are transmitted
--when the NewPopIn signal goes high highest cost first. Then these transmitted elements are eliminated from the
--the sort and the new elements are sorted until the NewPopIn signal goes high again.


architecture rtl of InsertionSort is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   signal AddressOutFromSortRegister : std_logic_vector (6 downto 0);
   --block memory signals
   signal WriteEnable : std_logic;
   signal WriteAddr   : std_logic_vector(6 downto 0) ;
   signal WriteData   : std_logic_vector(179 downto 0);
   signal ReadAddr    : std_logic_vector(6 downto 0);
   signal ReadData    : std_logic_vector(179 downto 0);
   --delay registers
   signal DataValidReg    : std_logic_vector (2 downto 0);
   signal HighAddressWr   : std_logic;
   signal LastTransmittingRes : std_logic;
   signal TransmittingResult  : std_logic;

   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin
   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   WriteEnable <= '1';
   --write address comes from serial cost function. Data is written every cycle
   --Read address from the sorting register. It is read one location per
   --cycle, not all data are valid, a controlsignal are used to show
   --which data is valid.(DataValid)
   ReadAddr <= AddressOutFromSortRegister;
   WriteAddr <= InputWeightAddress; 

   --maping the bits of the weigts to the memory's write data.
   Couplings : for i in 0 to c_AT-1 generate
      Couplings2: for j in 0 to c_NumBitsLow-1  generate
         WriteData(c_NumBitsLow*i*2+j)       <= InputWeights(i).re(j);
         WriteData(c_NumBitsLow*(i*2+1) + j) <= InputWeights(i).im(j);
      end generate;
   end generate;
   
   
   --Conrol signals are delayed because of memory read delay
   DataValid    <= DataValidReg(2);
   ----------------------------------------------------------------------------
   -- Clocked processes
   ----------------------------------------------------------------------------
   
      p_MyProcess : process(Clk) is
   
 
 begin
      if rising_edge(Clk) then
         if Rst_n = '0' then
            DataValidReg <= (others => '0') ;
            HighAddressWr <= '0';
            OutputWeights <= (others => (others => (others => '0') ) ); 
            OutputWeights <= (others => (others => (others => '0') ) ); 
         else
            --translating from memorys ReadData back to weights
            for i in 0 to c_AT-1 loop
               for j in 0 to c_NumBitsLow-1  loop
                  OutputWeights(i).re(j) <= ReadData(c_NumBitsLow*i*2+j);       
                  OutputWeights(i).im(j) <= ReadData(c_NumBitsLow*(i*2+1) + j);
               end loop;
            end loop;
            
            --Creating dataValid register to delay data valid signal
               --because of memory read delay
            DataValidReg <= DataValidReg(1 downto 0) &  '0';
            if TransmittingResult = '1' then
               DataValidReg <= DataValidReg(1 downto 0) & '1';
            end if;
         end if; 
      end if;
   end process; 
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------
   i_BlockMemory : entity GENETICALGLIB.Blockmemory180x128(Blockmemory180x128_a)
      PORT MAP (
         clka =>  Clk,
         wea(0) => WriteEnable,
         addra => WriteAddr,
         dina =>  WriteData,
         clkb =>  Clk,
         addrb => ReadAddr,
         doutb => ReadData);
   
   i_SortingRegister : entity GENETICALGLIB.SortingRegister(rtl)
      port map (
      Rst_n                => Rst_n,
      Clk                  => Clk,
      AddressIn            => InputCostAddress,
      InputCost            => InputCost.re,
      NewPopIn             => NewPopIn,
      TransmittingResult   => TransmittingResult,
      AddressOut           => AddressOutFromSortRegister
   );
end rtl;

