--File: Mutation_rtl.vhd (vhdl)
--mutates one individual with latency of 2 clock cycles
--(c) Kongsberg Defence & Aerospace 2015
--$Id: Mutation_rtl.vhd 14777 2015-03-24 10:58:36Z mathiab $
--Author: Mathias By
--
--Description: 
--The Mutation module takes one individual and 1260 random bits as well as some mutation rates as a table of integers.
--The mutation rate starts as the first individual, then for every clock cycle the NewPopIn signal is high the mutaion rate
--turned to the next mutation rate of the integer table and then back to the first mutationrate again. With a latency of 2 the
--individual in (WeightsIn) is then mutated using the random bits and the mutation rate to obtain the result which is the mutated
--version of WeightsIn; WeightsOut. NewPopOut is set high when the set of data that came in with the NewPopIn high. This first
--set of data of the population is not mutated.


architecture rtl of Mutation is

   ----------------------------------------------------------------------------
   -- Types, constants and signals
   ----------------------------------------------------------------------------
   signal WeightOutReg : t_1xAT10; --register used to delay output weight to simplify timing
   signal NewPopOutReg : std_logic; --register used to delay NewPopOut to match weight output
   ----------------------------------------------------------------------------
   -- Procedures and functions
   ----------------------------------------------------------------------------

begin

   ----------------------------------------------------------------------------
   -- Combinatorial processes
   ----------------------------------------------------------------------------
   

   ----------------------------------------------------------------------------
   -- Clocked processes
   ----------------------------------------------------------------------------
   
   p_MyProcess : process(Clk) is
      variable v_MutateReal : std_logic;
      variable v_MutateImag : std_logic;
      variable v_MutationRateState : integer range 0 to c_NumOfPop;--state variable to keep track of which mutation rate to use
   begin
      if rising_edge(Clk) then
         if Rst_n = '0' then
            WeightOutReg <= (others => (others => (others => '0') ) );
            v_MutationRateState := c_NumOfPop -1; --v_MutationRateState change when ever the NewPopIn signal is high.
                                                      --this means that the mutation rate for the first population will be
                                                      --according to v_MutationRateState := 0.
            NewPopOutReg <= '0';
            NewPopOut   <= '0'; 
         else
            for i in 0 to c_AT-1 loop --for all taps
               for j in c_NumBitsLow -1 downto 0 loop --for all bits of each tap
                  
                  --reset mutate signals
                  v_MutateImag := '1';
                  v_MutateReal := '1';
                 
                  --according to mutation rate a number of random bits are and'ed togheter with the
                  --mutate bits. This means that the mutate bits are only '1' if all the random bits
                  --are '1'. Which give a mutation propablity for each bit of 1/(2^(mutation rate)).
                  if MutationRate(v_MutationRateState) = 4 then 
                     for k in 0 to 4-1 loop
                        v_MutateImag := v_MutateImag and RandomBits((i*10+j)*2*7+k);
                        v_MutateReal := v_MutateReal and RandomBits((i*10+j)*2*7+k+7);
                     end loop;   
                  elsif MutationRate(v_MutationRateState) = 5 then 
                     for k in 0 to 5-1 loop
                        v_MutateImag := v_MutateImag and RandomBits((i*10+j)*2*7+k);
                        v_MutateReal := v_MutateReal and RandomBits((i*10+j)*2*7+k+7);
                     end loop;   
                  elsif MutationRate(v_MutationRateState) = 6 then 
                     for k in 0 to 6-1 loop
                        v_MutateImag := v_MutateImag and RandomBits((i*10+j)*2*7+k);
                        v_MutateReal := v_MutateReal and RandomBits((i*10+j)*2*7+k+7);
                     end loop;   
                  else 
                     for k in 0 to 7-1 loop
                        v_MutateImag := v_MutateImag and RandomBits((i*10+j)*2*7+k);
                        v_MutateReal := v_MutateReal and RandomBits((i*10+j)*2*7+k+7);
                     end loop;   
                  end if;
                  
                  If NewPopIn = '1' then --When a new population arrives
                     --The first individual of each population is not mutated
                     WeightOutReg(i).im(j) <= WeightsIn(i).im(j);    
                     WeightOutReg(i).re(j) <= WeightsIn(i).re(j);    
                  else   
                     --All other individuals are mutated with the given propability
                     WeightOutReg(i).im(j) <= WeightsIn(i).im(j) xor v_MutateImag;    
                     WeightOutReg(i).re(j) <= WeightsIn(i).re(j) xor v_MutateReal;    
                  end if;
                  
               end loop;
            end loop;
            if NewPopIn = '1' then
               --the mutation rate is changed to the next mutation rate through the variable v_MutationRateState.
               if v_MutationRateState < c_NumOfPop-1 then
                  v_MutationRateState := v_MutationRateState + 1; 
               else
                  v_MutationRateState := 0;
               end if;
            end if;
            --Delaying NewPop signal through mutation module
            NewPopOut <= NewPopOutReg;
            NewPopOutReg <= NewPopIn;
            --Setting weights from register to output
            WeightsOut <= WeightOutReg;
         end if; 
      end if;
   end process; 
   
   ----------------------------------------------------------------------------
   -- Instantiations
   ----------------------------------------------------------------------------

end rtl;

